camera = {}
camera._x = 0
camera._y = 0
camera.scaleX = 1
camera.scaleY = 1
camera.rotation = 0

local w, h = love.graphics.getDimensions()

function camera:set() -- Scale, Translate, Rotate -> that order
	love.graphics.push()

	love.graphics.scale(1 / self.scaleX, 1 / self.scaleY)
	love.graphics.translate(-self._x, -self._y)

	-- Rotate around center of the screen
	local lx, ly = camera:getLookPosition()
	love.graphics.translate(lx, ly)
	love.graphics.rotate(-self.rotation)
	love.graphics.translate(-lx, -ly)
end

function camera:unset()
	love.graphics.pop()
end

function camera:move(dx, dy)
	self._x = self._x + (dx or 0)
	self._y = self._y + (dy or 0)
end

function camera:moveRelative(dx, dy) -- Move up, regardless of camera orientation [Adjusts for rotation]
	camera:move(dx * math.cos(self.rotation) - dy * math.sin(self.rotation), dy * math.cos(self.rotation) + dx * math.sin(self.rotation))
end

function camera:rotate(dr)
	self.rotation = self.rotation + dr
end

function camera:setAngle(a)
	self.rotation = a
end

function camera:getAngle()
	return self.rotation
end

function camera:getScale()
	return self.scaleX, self.scaleY
end

function camera:scale(sx, sy)
	sx = sx or 1
	self.scaleX = self.scaleX * sx
	self.scaleY = self.scaleY * (sy or sx)
end

function camera:setX(value)
	if self._bounds then
		self._x = math.clamp(value, self._bounds.x1, self._bounds.x2)
	else
		self._x = value
	end
end

function camera:setY(value)
	if self._bounds then
		self._y = math.clamp(value, self._bounds.y1, self._bounds.y2)
	else
		self._y = value
	end
end

function camera:setPosition(x, y)
	if x then self:setX(x) end
	if y then self:setY(y) end
end

function camera:lookAt(x, y)
	self:setPosition(x - w * 0.5 * self.scaleX, y - h * 0.5 * self.scaleY)
end

function camera:update(dt)
end

function camera:getLookPosition()
	return self._x + w * 0.5 * self.scaleX, self._y + h * 0.5 * self.scaleY
end

function camera:getBounds()
	return unpack(self._bounds)
end

function camera:setBounds(x1, y1, x2, y2)
	self._bounds = { x1 = x1, y1 = y1, x2 = x2, y2 = y2 }
end

function camera:mousePosition()
  return camera:screenToWorld(love.mouse.getPosition())
end

function camera:screenToWorld(x, y)
	local x, y = x * self.scaleX, y * self.scaleY

	local cx, cy = camera:getLookPosition()
	local rx = ((x + self._x - cx) * math.cos(camera.rotation) - (y + self._y - cy) * math.sin(camera.rotation))
	local ry = ((y + self._y - cy) * math.cos(camera.rotation) + (x + self._x - cx) * math.sin(camera.rotation))
	return rx + cx, ry + cy
end

function camera:worldToScreen(x, y)
	local cx, cy = camera:getLookPosition()
	
	local rx = (x - cx) * math.cos(-camera.rotation) - (y - cy) * math.sin(-camera.rotation)
	local ry = (y - cy) * math.cos(-camera.rotation) + (x - cx) * math.sin(-camera.rotation)
	return (rx + cx - self._x) / self.scaleX , (ry + cy - self._y) / self.scaleY
end
