function drawGrid()
	love.graphics.setColor(colour.red)
	love.graphics.setLineWidth(1)
	if gridToggle then
		for y = 1, mapH - 1 do
			love.graphics.line(0, y * tileSize, mapW * tileSize, y * tileSize)
		end
		for x = 1, mapW - 1 do
			love.graphics.line(x * tileSize, 0, x * tileSize, mapH * tileSize)
		end
	end
	love.graphics.setLineWidth(2)
end

function drawBase()
	love.graphics.setColor(colour.black)
	for y = 0, mapH - 1 do
		for x = 0, mapW - 1 do
			if checkMap(x, y) == 0 then
				love.graphics.rectangle("fill", x * tileSize, y * tileSize, tileSize, tileSize)
			end
		end
	end

	if gridToggle then drawGrid() end

	love.graphics.setColor(255, 255, 255)
	for y = 0, mapH - 1 do
		for x = 0, mapW - 1 do
			if checkMap(x, y) == 0 then
				if checkMap(x - 1, y) ~= 0 then
					love.graphics.line(x * tileSize, y * tileSize, x * tileSize, (y + 1) * tileSize)
				end

				if checkMap(x + 1, y) ~= 0 then
					love.graphics.line((x + 1) * tileSize, y * tileSize, (x + 1) * tileSize, (y + 1) * tileSize)
				end

				if checkMap(x, y - 1) ~= 0 then
					love.graphics.line(x * tileSize, y * tileSize, (x + 1) * tileSize, y * tileSize)
				end

				if checkMap(x, y + 1) ~= 0 then
					love.graphics.line(x * tileSize, (y + 1) * tileSize, (x + 1) * tileSize, (y + 1) * tileSize)
				end
			end
		end
	end
end

function drawAI()
	for y = 0, mapH - 1 do
		for x = 0, mapW - 1 do
			if checkMap(x, y, mapAI) == 0 then
				love.graphics.setColor(colour.greent)
			else
				love.graphics.setColor(colour.redt)
			end
			love.graphics.rectangle("fill", x * tileSize, y * tileSize, tileSize, tileSize)
		end
	end
end

function drawWall()
	for i,v in ipairs(wallTable) do
		love.graphics.setColor(colour.black)
		love.graphics.rectangle("fill", v.x, v.y, v.w, v.h)
		love.graphics.setColor(colour.white)
		love.graphics.rectangle("line", v.x, v.y, v.w, v.h)
	end

	if placingWall then
		local v = placingWall
		love.graphics.setColor(colour.black)
		love.graphics.rectangle("fill", math.min(v.x1, v.x2), math.min(v.y1, v.y2), math.abs(v.x1 - v.x2), math.abs(v.y1 - v.y2))
		love.graphics.setColor(colour.white)
		love.graphics.rectangle("line", math.min(v.x1, v.x2), math.min(v.y1, v.y2), math.abs(v.x1 - v.x2), math.abs(v.y1 - v.y2))
	end
end

function drawProp(bg)
	for i,v in ipairs(propTable) do
		love.graphics.setColor(colour.white)
		if editProp == v then love.graphics.setColor(colour.red) end

		if v.background == bg then
			love.graphics.draw(propList.imgs[v.img], v.x, v.y, v.r, v.scale or 1, v.scale or 1, v.w * 0.5, v.h * 0.5)
		end
	end
end

function drawEnem()
	for i,v in ipairs(enemyTable) do
		love.graphics.setColor(colour.white)
		if editEnem == v then love.graphics.setColor(colour.red) end

		love.graphics.draw(enemyList.imgs[v.img], v.x, v.y, v.r, 1, 1, v.w * 0.5, v.h * 0.5)
		love.graphics.line(v.x, v.y, v.x + math.cos(v.r) * 10, v.y + math.sin(v.r) * 10)
	end
end

function checkMap(x, y, t)
	t = t or mapBase
	if t[y] then
		if t[y][x] then
			return t[y][x]
		else
			return -1
		end
	end

	return -2
end

function snapToVal(v, n)
	n = n or snapSize
	return math.floor(v/n)*n
end

colour = {
	["white"] 	= {255, 	255, 	255},
	["black"] 	= {0,		0, 		0  },
	["red"]	  	= {180, 	100, 	100},
	["green"] 	= {100, 	180, 	100},
	["redt"]  	= {180, 	100, 	100, 	60},
	["greent"] 	= {100, 	180, 	100, 	60},
}