local nk = require 'nuklear'
local binser = require 'binser'

require "camera"
require "draw"
require "add"

local w, h = love.graphics.getDimensions()

love.window.showMessageBox("Level Editor Disclaimer", "This editor is meant for developer use only at the moment, hence - it is not well documented.\nHowever, all levels in Recalling Monochrome were created using this tool and it is fully functioning.", "warning", true)

mapW, mapH, lockedDim = 16, 16, false
tileSize = 32
snapSize = 8
mapBase 	= {} -- 2D
mapAI 		= {} -- 2D
wallTable 	= {}
propTable 	= {}
enemyTable 	= {}
started = false

local sp = love.graphics.newImage("spawn.png")

-- Views
gridToggle = false
baseToggle = true
aiToggle   = false
wallToggle = true
propToggle = true
enemToggle = true

placingWall = nil
selectedProp = nil
editProp = nil
selectedEnem = nil
editEnem = nil

plottingPath = false

playerStartingPoint = {x = 16, y = 16}

cutsceneText = {value = ""};

names = {
	level = {value = ""},
	map = {value = ""},

	url = function(self)
		return self.level.value.."/"..self.map.value.."/"
	end,
}

function save()
	local str = binser.serialize(mapBase, mapAI, wallTable, propTable, enemyTable, mapW, mapH, playerStartingPoint, cutsceneText.value)
	local file = love.filesystem.newFile(names:url().."data")
	file:open("w")
	file:write(str)
	file:close()
end

function load()
	local str = love.filesystem.read(names:url().."data")
	mapBase, mapAI, wallTable, propTable, enemyTable, mapW, mapH, playerStartingPoint, cutsceneText = binser.deserializeN(str, 9)
	cutsceneText = {value = cutsceneText}

	getImgs(names:url().."props", propList)
	getImgs(names:url().."enemies", enemyList)

	if love.filesystem.isFile(names:url().."background.png") then
		background = love.graphics.newImage(names:url().."background.png")
	end
end

-- Edit Mode
edit = {value = 1, items = {"base", "ai", "wall", "prop", "enem"}, selected = function(self) return self.items[self.value] end}
currentLevels = {value = 1, items = {}, selected = function(self) return self.items[self.value] end}
currentMaps = {value = 1, items = {}, selected = function(self) return self.items[self.value] end}

propList = {value = 1, items = {}, imgs = {}, selected = function(self) return self.items[self.value] end}
enemyList = {value = 1, items = {}, imgs = {}, selected = function(self) return self.items[self.value] end}

local usingExisting = false

function love.load()
	love.keyboard.setKeyRepeat(true)
	nk.init()

	getDir("", currentLevels)

	arrow = love.graphics.newImage("arrow.png")
end

function getDir(s, t)
	t.items = {}

	local lfs = love.filesystem
	local filesTable = lfs.getDirectoryItems(s)
	for i,v in ipairs(filesTable) do
		if lfs.isDirectory(s.."/"..v) then
			t.items[#t.items + 1] = v
		end
	end
end

function getImgs(s, t)
	t.items = {}
	t.imgs = {}

	local lfs = love.filesystem
	local filesTable = lfs.getDirectoryItems(s)
	for i,v in ipairs(filesTable) do
		if lfs.isFile(s.."/"..v) then
			t.items[#t.items + 1] = v
			t.imgs[#t.imgs + 1] = love.graphics.newImage(s.."/"..v)
		end
	end
end

function love.update(dt)
	nk.frameBegin()
	if nk.windowBegin('Settings', 0, 0, 200, 450,
			'border', 'title', 'movable', 'minimizable', 'scrollbar', 'scalable') then
		if not lockedDim then
			nk.layoutRow('dynamic', 30, 1)
			nk.label('Level Name')
			local state = nk.edit('simple', names.level)
			if state == "deactivated" then
				getDir(names.level.value, currentMaps)
			end

			nk.layoutRow('dynamic', 30, {0.32, 0.68})
			nk.label('Existing')
			if nk.combobox(currentLevels, currentLevels.items) then
				names.level.value = currentLevels:selected()
				getDir(names.level.value, currentMaps)
			end

			-- Make it change to Load Map instead of New Map if an existing map is selected
			nk.layoutRow('dynamic', 30, 1)
			nk.label('Map Name')
			local _, changed = nk.edit('simple', names.map)
			if changed then
				usingExisting = false

				for i=1,#currentMaps.items do
					if currentMaps.items[i] == names.map.value then
						usingExisting = true
					end
				end
			end
			nk.layoutRow('dynamic', 30, {0.32, 0.68})
			nk.label('Existing')
			if nk.combobox(currentMaps, currentMaps.items) then
				names.map.value = currentMaps:selected()
				usingExisting = true
			end

			nk.layoutRow('dynamic', 30, 1)
			nk.label('Map Size')
			mapW = nk.property("Width", 1, mapW, 44, 1, 1)
			mapH = nk.property("Height", 1, mapH, 32, 1, 1)
		end

		nk.layoutRow('dynamic', 30, 1)
		if nk.button('Toggle Lock') then
			lockedDim = not lockedDim
		end

		if not lockedDim then
			if nk.button(string.format("%s Map", usingExisting and "Load" or "New")) then
				lockedDim = true
				started = true

				if usingExisting then
					load()
					camera:lookAt(mapW * 0.5 * tileSize, mapH * 0.5 * tileSize)
				else
					local success = love.filesystem.createDirectory(names.level.value.."/"..names.map.value.."/")
					if success then
						for y=0, mapH - 1 do
							mapBase[y] = {}
							mapAI[y] = {}
							for x=0, mapW - 1 do
								mapBase[y][x] = 0
								mapAI[y][x] = 0
							end
						end

						camera:lookAt(mapW * 0.5 * tileSize, mapH * 0.5 * tileSize)
					else
						started = false
						lockedDim = false
					end
				end
			end
		end

		nk.layoutRow('dynamic', 30, 1)
		nk.label('Placement')
		snapSize = nk.property("Snap to", 2, snapSize, tileSize, 2, 2)

		nk.layoutRow('dynamic', 30, 1)
		nk.label('Export')
		if nk.button('Save') then
			save()
		end
		if nk.button('Render') then
			--
		end
	end
	nk.windowEnd()

	if nk.windowBegin('View and Edit Modes', w - 250, 0, 250, 220,
			'border', 'title', 'movable', 'minimizable', 'scrollbar', 'scalable') then
		nk.layoutRow('dynamic', 30, 1)
		nk.label('View Modes')
		nk.layoutRow('dynamic', 30, 2)
		gridToggle = not nk.checkbox("Grid", not gridToggle)
		baseToggle = not nk.checkbox("Base", not baseToggle)
		aiToggle = not nk.checkbox("AI-Walkable", not aiToggle)
		wallToggle = not nk.checkbox("Walls", not wallToggle)
		propToggle = not nk.checkbox("Props", not propToggle)
		enemToggle = not nk.checkbox("Enemies", not enemToggle)

		nk.layoutRow('dynamic', 30, 2)
		nk.label('Edit Mode')
		if nk.combobox(edit, edit.items) then
			if edit:selected() == "ai" then
				aiToggle = true
			end
			editProp = nil
			editEnem = nil
		end
	end
	nk.windowEnd()

	if nk.windowBegin('Props', w - 200, h - 220, 200, 150,
			'border', 'title', 'movable', 'minimizable', 'scrollbar', 'scalable') then

		nk.layoutRow('dynamic', 30, 1)
		nk.label("Prop to Place")
		if nk.combobox(propList, propList.items) then
		end

		if nk.button("Create Prop") then
			addProp(propList.value, 16, 16, 0)
		end

		if nk.button('Refresh List') then
			getImgs(names:url().."props", propList)
		end
	end
	nk.windowEnd()

	if nk.windowBegin('Enemies', w - 200, h - 370, 200, 150,
			'border', 'title', 'movable', 'minimizable', 'scrollbar', 'scalable') then

		nk.layoutRow('dynamic', 30, 1)
		nk.label("Enemy to Place")
		if nk.combobox(enemyList, enemyList.items) then
		end

		if nk.button("Create Enemy") then
			addEnemy(enemyList.value, 16, 16)
		end

		if nk.button('Refresh List') then
			getImgs(names:url().."enemies", enemyList)
		end
	end
	nk.windowEnd()

		if nk.windowBegin('Selected Object', 200, 0, 150, 200,
				'border', 'title', 'movable', 'minimizable', 'scrollbar', 'scalable') then
			
			if editProp then
				-- edit if prop or push and whether to render in the background or not
				nk.layoutRow('dynamic', 30, 1)
				nk.label(propList.items[editProp.img])
				editProp.r = nk.property("Rot", -math.pi*2, editProp.r, math.pi*2, math.pi/8, math.pi/8)
				editProp.type = nk.checkbox("Solid", editProp.type == "prop") and "prop" or "push"
				editProp.ttype = nk.checkbox("Movable", editProp.ttype == "immovable") and "immovable" or "movable"

				if type(editProp.hueCycle) == "nil" then editProp.hueCycle = false end

				editProp.hueCycle = not nk.checkbox("Cycle Hue", not editProp.hueCycle)

				if editProp.type == "prop" then
					editProp.background = not nk.checkbox("In background", not editProp.background)
				end
			elseif editEnem then
				-- edit if prop or push and whether to render in the background or not
				nk.layoutRow('dynamic', 30, 1)
				nk.label("Enemy")
				editEnem.r = nk.property("Rot", -math.pi*2, editEnem.r, math.pi*2, math.pi/8, math.pi/8)
				editEnem.mode = nk.checkbox("Partolling", editEnem.mode == "stand") and "stand" or "patrol"
				editEnem.loop = nk.checkbox("Loop Patrol", editEnem.loop == "reverse") and "reverse" or "circle"

				nk.layoutRow('dynamic', 30, 2)
				nk.label("Weapon")
				editEnem.weapon = guns[nk.combobox(guns[editEnem.weapon], {"knife", "pistol", "auto", "shotgun"})]
			end
		end
		nk.windowEnd()

	if nk.windowBegin('Player Spawn Point', 350, 0, 150, 120,
			'border', 'title', 'movable', 'minimizable', 'scrollbar', 'scalable') then
			nk.layoutRow('dynamic', 30, 1)
			playerStartingPoint.x = nk.property("X", 0, playerStartingPoint.x, mapW * tileSize, snapSize, snapSize)
			playerStartingPoint.y = nk.property("Y", 0, playerStartingPoint.y, mapH * tileSize, snapSize, snapSize)
	end
	nk.windowEnd()

	if nk.windowBegin('Cutscene (Level Start)', w - 250, 220, 250, 130,
			'border', 'title', 'movable', 'minimizable', 'scrollbar', 'scalable') then

			nk.layoutRow('dynamic', 100, 1)
			nk.edit('box', cutsceneText)
	end
	nk.windowEnd()
	nk.frameEnd()

	local x, y = camera:mousePosition()
	local mx, my = math.floor(x / tileSize), math.floor(y / tileSize)
	if edit:selected() == "base" then
		if checkMap(mx, my) >= 0 and love.mouse.isDown(1, 2) then
			mapBase[my][mx] = love.mouse.isDown(1) and 0 or 1
			mapAI[my][mx] = mapBase[my][mx]
		end
	elseif edit:selected() == "ai" then
		if checkMap(mx, my) >= 0 and love.mouse.isDown(1, 2) then
			mapAI[my][mx] = love.mouse.isDown(1) and 0 or 1
		end
	elseif edit:selected() == "wall" then
		if placingWall then
			placingWall.x2, placingWall.y2 = snapToVal(x), snapToVal(y)
		end
	elseif edit:selected() == "enem" then
		if plottingPath then
			local i = #editEnem.path
			local needToPlace = (i == 0) or (editEnem.path[i].x ~= (mx + 0.5) * tileSize or editEnem.path[i].y ~= (my + 0.5) * tileSize)

			if needToPlace and checkMap(mx, my, mapAI) == 0 then
				editEnem.path[#editEnem.path + 1] = {x = (mx + 0.5) * tileSize, y = (my + 0.5) * tileSize}
			end
		end
	end
end

function love.draw()
	if started then
		if background then
			for x = 0, w / tileSize do
				for y = 0, h / tileSize do
					love.graphics.draw(background, x * tileSize, y * tileSize)
				end
			end
		end
	end

	camera:set()
		if started then
			if baseToggle then drawBase() end
			if aiToggle then drawAI() end
			if propToggle then drawProp(true) end
			if enemToggle then drawEnem() end
			if wallToggle then drawWall() end
			if propToggle then drawProp(false) end
		end

		if editEnem then
			for i = 2, #editEnem.path do
				local v = editEnem.path[i]
				love.graphics.line(editEnem.path[i-1].x, editEnem.path[i-1].y, v.x, v.y)
				love.graphics.draw(arrow, (editEnem.path[i-1].x + v.x) * 0.5, (editEnem.path[i-1].y + v.y) * 0.5 , math.atan2(v.y - editEnem.path[i-1].y, v.x - editEnem.path[i-1].x), 1, 1, arrow:getWidth() * 0.5, arrow:getHeight() * 0.5)
				--love.graphics.rectangle("fill", v.x * tileSize - 2, v.y * tileSize - 2, 4, 4)
			end
		end

		love.graphics.setColor(colour.white)
		love.graphics.draw(sp, playerStartingPoint.x, playerStartingPoint.y, 0, 1, 1, sp:getWidth() * 0.5, sp:getHeight() * 0.5)

		local x, y = camera:mousePosition()
		local sv = (edit:selected() == "base" or edit:selected() == "ai") and tileSize or snapSize
		local mx, my = snapToVal(x, sv), snapToVal(y, sv)
		if edit:selected() == "prop" or edit:selected() == "enem" then
			--local img = propList.imgs[propList.value]
			--love.graphics.draw(img, mx - img:getWidth() * 0.5, my - img:getHeight() * 0.5)
			local x, y = camera:mousePosition();
			if selectedProp then
				selectedProp.x, selectedProp.y = snapToVal(x, snapSize), snapToVal(y, snapSize)
				local v = selectedProp
				love.graphics.draw(propList.imgs[v.img], v.x, v.y, v.r, v.scale or 1, v.scale or 1, v.w * 0.5, v.h * 0.5)
			end
			if selectedEnem then
				selectedEnem.x, selectedEnem.y = snapToVal(x, snapSize), snapToVal(y, snapSize)
				local v = selectedEnem
				love.graphics.draw(enemyList.imgs[v.img], v.x, v.y, v.r, 1, 1, v.w * 0.5, v.h * 0.5)
			end
		else
			love.graphics.rectangle("line", mx, my, sv, sv)
		end
	camera:unset()

	nk.draw()

	love.graphics.print(string.format("%1.f %1.f", camera:mousePosition()))
end

function love.keypressed(key, scancode, isrepeat)
	if key == "escape" then love.event.quit() end

	if nk.keypressed(key, scancode, isrepeat) then
		return -- event consumed
	end
end

function love.keyreleased(key, scancode)
	if nk.keyreleased(key, scancode) then
		return -- event consumed
	end
end

function love.mousepressed(x, y, button, istouch)
	if nk.mousepressed(x, y, button, istouch) then
		return -- event consumed
	end

	if started then
		local x, y = camera:mousePosition();
		if edit:selected() == "wall" then
			x, y = snapToVal(x), snapToVal(y)
			if button == 1 then
				placingWall = {x1 = x, y1 = y, x2 = x, y2 = y}
			elseif button == 2 then
				for i,v in ipairs(wallTable) do
					if x >= v.x and y >= v.y and x < v.x + v.w and y < v.y + v.h then
						table.remove(wallTable, i)
						break
					end
				end
			end
		end

		if edit:selected() == "prop" then
			for i,v in ipairs(propTable) do
				if x >= v:xx() and y >= v:yy() and x < v:xx() + v.w and y < v:yy() + v.h then
					if button == 1 then
						selectedProp = v
						editProp = v
						table.remove(propTable, i)
					elseif button == 2 then
						table.remove(propTable, i)
					end
					goto cont
				end
			end
			editProp = nil
			::cont::
		end

		if edit:selected() == "enem" then
			if button == 1 and not plottingPath then
				for i,v in ipairs(enemyTable) do
					if x >= v:xx() and y >= v:yy() and x < v:xx() + v.w and y < v:yy() + v.h then
						selectedEnem = v
						editEnem = v
						table.remove(enemyTable, i)
						goto cont
					end
				end
				editEnem = nil
				::cont::
			elseif button == 2 and editEnem then
				for i,v in ipairs(enemyTable) do
					if x >= v:xx() and y >= v:yy() and x < v:xx() + v.w and y < v:yy() + v.h then
						if v == editEnem then
							editEnem.path = {}
							plottingPath = true
						end
					end
				end
			end
		end
	end
end

function love.mousereleased(x, y, button, istouch)
	if nk.mousereleased(x, y, button, istouch) then
		return -- event consumed
	end

	local x, y = camera:mousePosition();
	if edit:selected() == "wall" then
		if button == 1 and placingWall then
			placingWall.x2, placingWall.y2 = snapToVal(x), snapToVal(y)

			local v = placingWall
			if not (math.abs(v.x1 - v.x2) == 0 or math.abs(v.y1 - v.y2) == 0) then
				table.insert(wallTable, {x = math.min(v.x1, v.x2), y = math.min(v.y1, v.y2), w = math.abs(v.x1 - v.x2), h = math.abs(v.y1 - v.y2)})
			end

			placingWall = nil
		elseif button == 2 then
		end
	end

	if edit:selected() == "prop" and selectedProp and button == 1 then
		table.insert(propTable, selectedProp)
		selectedProp = nil
	end

	if edit:selected() == "enem" and selectedEnem and button == 1 then
		table.insert(enemyTable, selectedEnem)
		selectedEnem = nil
	end

	if edit:selected() == "enem" and editEnem and button == 2 and plottingPath then
		plottingPath = false
	end
end

function love.mousemoved(x, y, dx, dy, istouch)
	if nk.mousemoved(x, y, dx, dy, istouch) then
		return -- event consumed
	end

	if love.mouse.isDown(3) then
		camera:moveRelative(-dx, -dy)
	end
end

function love.textinput(text)
	if nk.textinput(text) then
		return -- event consumed
	end
end

function love.wheelmoved(x, y)
	if nk.wheelmoved(x, y) then
		return -- event consumed
	end
end