local bloodr = 8
--local bloodImg = love.graphics.newImage("img/blood.png")
--bloodImg:setFilter("nearest", "nearest")
local sp = 100
local bloodCanvas = love.graphics.newCanvas(sp * 3, sp * 3)

props = {}
local pImg = love.graphics.newImage("img/player.png")

local proto = {
	x = 0, y = 0, r = 0, 
	w = 24,
	h = 24,

	xx = function(self)
		return self.x - self.w * 0.5
	end,
	yy = function(self)
		return self.y - self.h * 0.5
	end,

	type = "prop", -- type prop does not interact with npcs; push does
	ttype = "movable",

	vel = 300,
	update = true,

	img = pImg,
	isGun = false,
	ammo = 0,
	btm = 0,
	shrinking = false,
	scale = 1,

	background = true,

	displayR = nil,

	hueCycle = false,
	color = 0,

	c = {255, 255, 255}
}

local mt = {
	__index = proto
}

function addProp(vars, weaponTable)
	local set = {}
	setmetatable(set, mt)

	for k, v in pairs(vars) do
		set[k] = v
	end

	set.w = weaponTable and 16 or (set.w or 24)
	set.h = weaponTable and 16 or (set.h or 24)

	set.img = weaponTable and guns[weaponTable[1]][1] or (type(set.img) == "number" and propList[set.img] or set.img)
	set.isGun = weaponTable and weaponTable[1] or false
	set.ammo = weaponTable and weaponTable[2] or 0
	set.btm = weaponTable and guns[weaponTable[1]][2] or 0

	if set.hueCycle then
		set.color = love.math.random(0, 255)
	end

	table.insert(props, set)
	
	local v = props[#props]
	world:add(v, v:xx(), v:yy(), v.w, v.h)

	if v.ttype == "immovable" then v.update = false end

	local dist = love.math.random(20, 40)
	flux.to(v, 1, {vel = 0}):ease("quartout")
end

function updateProps(dt)
	for i,v in ipairs(props) do
		if v.hueCycle then
			v.color = (v.color + dt * 255) % 256
		end

		if v.update then
			if v.vel == 0 then
				v.update = false
			end
			moveProp(v, math.cos(v.r) * v.vel, math.sin(v.r) * v.vel, dt)
		end
		if v.isObjective then
			local dir = v.shrinking and -1 or 1

			v.scale = v.scale + dir * dt

			v.displayR = (v.displayR + math.pi * dt) % (math.pi * 2)

			if v.scale >= 1.5 or v.scale < 1 then
				v.shrinking = not v.shrinking
			end
		end
	end
end

function drawProps(bg)
	for i,v in ipairs(props) do
		if v.background == bg then
			if v.hueCycle then
				--love.graphics.setColor(HSL(v.color, 255, 128, 60))
				love.graphics.setColor(225, 225, 255, 60 + ((v.color-127) / 128) * 5)
			else
				love.graphics.setColor(v.c)
			end
			love.graphics.draw(v.img, v.x, v.y, v.displayR or v.r, v.scale or 1, v.scale or 1, v.w * 0.5, v.h * 0.5)
		end
	end
end

function splatter(x, y, nmin, nmax)
	blood(nmin, nmax)

	love.graphics.setCanvas(back.c)
		local parallaxStrength = 0.3
		local mx, my = mapW * tileSize * 0.5, mapH * tileSize * 0.5
		local nx, ny = x - mx - (player.x - mx) * parallaxStrength , y - my - (player.y - mx) * parallaxStrength

		local rx = nx * math.cos(-back.r) - ny * math.sin(-back.r)
		local ry = ny * math.cos(-back.r) + nx * math.sin(-back.r)

		
		love.graphics.push()
			if not nmin then
				love.graphics.setBlendMode("multiply")
			end
			love.graphics.translate(rx + back.c:getWidth() * 0.5, ry + back.c:getHeight() * 0.5)
			love.graphics.draw(bloodCanvas, -bloodCanvas:getWidth() * 0.5, -bloodCanvas:getHeight() * 0.5)
			love.graphics.setBlendMode("alpha")
		love.graphics.pop()
	love.graphics.setCanvas()
end

function blood(nmin, nmax)
	local n1, n2 = nmin or 50, nmax or 100 
	local c = cRef[love.math.random(1, 7)]

	love.graphics.setCanvas(bloodCanvas)
		if nmin then
			love.graphics.setColor(255, 255, 255)
			love.graphics.clear(255, 255, 255, 0)
		else
			love.graphics.setColor(color[c])
			love.graphics.clear(255, 255, 255, 255)
		end
		
		love.graphics.translate(bloodCanvas:getWidth() * 0.5, bloodCanvas:getHeight() * 0.5)
		
		for i = 1, love.math.random(n1, n2) do
			local s = love.math.random(0.5, 2)

			love.graphics.circle("fill", love.math.random(-sp, sp), love.math.random(-sp, sp), s * bloodr)
			--love.graphics.draw(bloodImg, love.math.random(-sp, sp), love.math.random(-sp, sp), 0, s, s, bloodImg:getWidth() * 0.5, bloodImg:getHeight() * 0.5)
		end
		love.graphics.translate(-bloodCanvas:getWidth() * 0.5, -bloodCanvas:getHeight() * 0.5)
	love.graphics.setCanvas()

	love.graphics.setColor(255, 255, 255)
end

color = {
	white 		= {245, 245, 245},
	pure 		= {255, 255, 255},

	red 		= {230, 60, 60},
	orange 		= {230, 120, 30},
	yellow 		= {230, 230, 60},
	green 		= {120, 230, 60},
	blue 		= {60, 120, 230},
	indigo		= {120, 120, 230},
	violet		= {120, 30, 120},

	black 		= {15, 15, 15},
	grey 		= {25, 25, 26},
	lightgrey 	= {155, 175, 155},
}

cRef = {"red", "orange", "yellow", "green", "blue", "indigo", "violet"}

function HSL(h, s, l, a)
	if s<=0 then return l,l,l,a end
	h, s, l = h/256*6, s/255, l/255
	local c = (1-math.abs(2*l-1))*s
	local x = (1-math.abs(h%2-1))*c
	local m,r,g,b = (l-.5*c), 0,0,0
	if h < 1     then r,g,b = c,x,0
	elseif h < 2 then r,g,b = x,c,0
	elseif h < 3 then r,g,b = 0,c,x
	elseif h < 4 then r,g,b = 0,x,c
	elseif h < 5 then r,g,b = x,0,c
	else              r,g,b = c,0,x
	end return {(r+m)*255,(g+m)*255,(b+m)*255,a}
end