en = {}

local pointerImg 	= love.graphics.newImage("img/misc/arrow.png")
local alertImg		= love.graphics.newImage("img/misc/alert.png")
stunImg		= love.graphics.newImage("img/misc/stun.png")
eyeImg		= love.graphics.newImage("img/misc/eye.png")

local automataBase = love.graphics.newImage("img/tutorial/automataBase.png")
local automataBlinker = love.graphics.newImage("img/tutorial/automataBlinker.png")

en.proto = {
	x = 0,
	y = 0,

	w = 24,
	h = 24,

	xx = function(self)
		return self.x - self.w * 0.5
	end,
	yy = function(self)
		return self.y - self.h * 0.5
	end,

	speed = 50,
	pace = 50,
	run = 100,
	r = 0,

	img = love.graphics.newImage("img/player.png"),

	heading = nil,
	path = {},
	pathIndex = 1,

	fov = math.rad(120),
	foresight = love.graphics.getHeight() * 0.7,
	mode = "patrol",

	type = "push",
	ttype = "enemy",

	timingBetweenNodes = 0,

	patrolAround = nil,
	loop = "reverse",

	scaleArrow = 1,
	signArrow = 1,

	bt = 0,
	btm = 0.5,

	embarkType = nil,

	stuckTime = 0,

	alertScale = 0,

	slash = nil,
	slashr = math.rad(90) * 0.5,

	weapon = nil, -- nil gun means knife only
	ammo = 0,

	stunned = 0, -- 0 is not stunned. It decays over time

	v = {x = 0, y = 0}, -- added velocity from explosions/such

	color = 0,
	tutorial = false,
}

en.mt = {
	__index = en.proto
}

enBullets = {}

function en.canHearPlayer(n, r)
	en.canHear(n, player.x, player.y, r)
end

function en.canHear(n, x, y, r)
	local range = r or 400

	for i,v in ipairs(n) do
		if math.dist(v.x, v.y, x, y) <= range and v.embarkType ~= "noise" and not en.canSeePlayer(v) then
			if not v.tutorial then
				v.patrolAround = nil
				local p = generatePath(math.floor(v.x / tileSize), math.floor(v.y / tileSize), math.floor(x / tileSize), math.floor(y / tileSize))
				en.embark(v, p, v.run, nil, "noise")
			else
				flux.to(v, 1, {r = math.angle(v.x, v.y, x, y)}):ease("linear")
			end
		end
	end
end

function en.deadMan(k, n)
	local range = 300

	for i,v in ipairs(n) do
		if math.dist(v.x, v.y, k.x, k.y) <= range then
			if not v.tutorial then
				v.patrolAround = nil
				local p = generatePath(math.floor(v.x / tileSize), math.floor(v.y / tileSize), math.floor(k.x / tileSize), math.floor(k.y / tileSize))
				en.embark(v, p, v.run)
			end
		end
	end
end

function drawBullets()
	love.graphics.setColor(255, 255, 255)
	for i,v in ipairs(enBullets) do
		love.graphics.draw(bulletImg, v.x, v.y, v.a + math.pi * 0.5, 1, 1, bulletImg:getWidth() * 0.5, bulletImg:getHeight() * 0.5)
	end
end

function updateBullets(dt)
	for i,v in ipairs(enBullets) do
		if player.overlayOp ~= 0 then break end

		local ox, oy = v.x, v.y
		v.x = v.x + math.cos(v.a) * v.v * dt
		v.y = v.y + math.sin(v.a) * v.v * dt

		local tt = true

		if player.abilityInv[player.abilitySelected] then
			if player.abilityInv[player.abilitySelected][1] == "psyche wall" and player.abilityInv[player.abilitySelected][2] then
				if math.dist(v.x, v.y, player.x, player.y) <= 28 then
					table.remove(enBullets, i)
					playSound("perished")

					for i=1,4 do
						newParticle(v.x, v.y, math.random(0.9, 1.1), math.pi, {v.a, v.v}, 0.1)
					end

					tt = false
				end
			end
		end

		if tt then
			if checkMap(math.floor(v.x / tileSize), math.floor(v.y / tileSize)) ~= 0 then
				--splatter(v.x, v.y, 20, 40)
				table.remove(enBullets, i)
			else
				local items = world:querySegment(ox, oy, v.x, v.y, function(item) return item == player or item.type == "wall" or (item.hueCycle and item.type == "push") end)
				--local items = world:queryPoint(v.x, v.y, function(item) return item ~= player end)
				if #items > 0 then
					table.remove(enBullets, i)

					--playSound("perished")
					if items[1] == player then
						if IS_TUTORIAL then
							player.abilityDrain = player.abilityDrain + 0.35

							if player.abilityDrain >= 1 then
								killPlayer(player, 0, 0)
							end
						else
							killPlayer(items[1], v.a)
						end
					end

					if (items[1].hueCycle and items[1].type == "push") then
						playSound("glassbreak")
						removeElement(props, items[1])
						world:remove(items[1])
					end
					return
				end
			end
		end
	end
end

function en.canSeePlayer(v)
	if math.abs(angleDifference(v.r, math.angle(v.x, v.y, player.x, player.y))) < v.fov * 0.5 and math.dist(v.x, v.y, player.x, player.y) <= v.foresight then
		local items, len = world:querySegment(v.x, v.y, player.x, player.y, function(item) return item.ttype == "player" or item.type == "wall" end)
		if items[1] == player then
			return true
		end
	end

	return false
end

function en.canSeeExplosion(v, x, y)
	local items, len = world:querySegment(v.x, v.y, x, y, function(item) return item.type == "wall" end)
	if len == 0 then
		return true
	end
	return false
end

function en.alertOthers(k, r)
	local range = r or 150

	for i,v in ipairs(enemies) do
		if math.dist(v.x, v.y, k.x, k.y) <= range and v ~= k then
			if not v.tutorial then en.chase(v) end
			--v.mode = "patrol"
			--v.timingBetweenNodes = 0

			--v.patrolAround = nil
			--local p = generatePath(math.floor(v.x / tileSize), math.floor(v.y / tileSize), math.floor(player.x / tileSize), math.floor(player.y / tileSize))
			--en.embark(v, p, v.run)

			--v.bt = 0
		end
	end
end

function en.canBeSeen(v)
	if math.abs(angleDifference(player.r, math.angle(player.x, player.y, v.x, v.y))) < v.fov * 0.5 then --item.type ~= "prop" and item.ttype ~= "player"
		local items, len = world:querySegment(player.x, player.y, v.x, v.y, function(item) return item.type == "wall" or item.ttype == "enemy" end)
		if items[1] == v then
			return true
		end
	end

	return false
end

function en.new(vars)
	local set = {}
	setmetatable(set, en.mt)

	for k, v in pairs(vars) do
		set[k] = v
	end

	world:add(set, set:xx(), set:yy(), set.w, set.h)
	set.color = love.math.random(0, 255)

	return set
end

function en.die(m, a, remEl, notSignal)
	if not m.tutorial then splatter(m.x, m.y) end

	if not m.tutorial then
		playSound("perished")
		addProp({x = m.x, y = m.y, r = a, c = HSL(m.color, 255, 128), vel = (not remEl) and 600 or 300})

		local choice = love.math.random(1, 6)
		--[[if choice == 1 then
			addProp(m.x, m.y, a, 100, {"auto", love.math.random(3, 14)})
		elseif choice == 2 then
			addProp(m.x, m.y, a, 100, {"pistol", love.math.random(1, 7)})
		elseif choice == 3 then
			addProp(m.x, m.y, a, 100, {"shotgun", love.math.random(1, 10)})
		end]]
		if choice <= 2 and m.weapon ~= "knife" and m.weapon then
			addProp({x = m.x, y = m.y, r = a, vel = 100}, {m.weapon, m.ammo})
		end
	else
		playSound("glassbreak")
		addProp({x = m.x, y = m.y, r = 0, img = automataBase, vel = 0})
	end	

	if remEl then
		removeElement(enemies, m)
		if not notSignal then en.deadMan(m, enemies) end
	end
	camera:split(8, 0.05)
	world:remove(m)
end

function en.embark(v, path, speed, loop, type)
	v.loop = false or loop
	v.speed = speed or v.speed

	v.mode = "patrol"
	v.path = table.clone(path)
	v.pathIndex = 1
	v.heading = v.path[v.pathIndex]
	v.timingBetweenNodes = 0
	v.embarkType = type
end

function en.chase(v)
	v.mode = "chase"
	v.speed = v.run
	v.timingBetweenNodes = 0
	v.patrolAround = nil

	flux.to(v, 0.2, {alertScale = 1}):ease("quadin")
end

function en.stun(v, t)
	v.stunned = t or 10

	v.heading = nil
	v.path = nil
	v.pathIndex = 1
	v.speed = v.pace * 0.5

	v.mode = "standingAround"
	v.speed = v.pace
	v.patrolAround = {x = math.floor(v.x / tileSize), y = math.floor(v.y / tileSize)}
end

function en.update(v, dt)
	local canSee = en.canSeePlayer(v)

	v.color = (v.color + dt * 255 * 0.5) % 256

	if v.stunned > 0 then canSee = false end
	v.stunned = math.max(0, v.stunned - dt)

	if v.mode == "stand" then
		if canSee then
			if not v.tutorial then
				en.chase(v)
				en.alertOthers(v)
			else
				en.alertOthers(v)
			end
		end
	end

	if v.mode == "standingAround" then
		if canSee then
			if not v.tutorial then
				en.chase(v)
				en.alertOthers(v)
			else
				en.alertOthers(v)
			end
		else
			local dx, dy = love.math.random(-2, 2), love.math.random(-2, 2)

			local vx, vy = math.floor(v.x / tileSize), math.floor(v.y / tileSize)
			local p = generatePath(vx, vy, v.patrolAround.x + dx, v.patrolAround.y + dy)
			while (not p) do
				local dx, dy = love.math.random(-2, 2), love.math.random(-2, 2)
				p = generatePath(vx, vy, v.patrolAround.x + dx, v.patrolAround.y + dy)
			end

			en.embark(v, p)
		end
	end

	if v.heading and v.mode == "patrol" then
		local ang = math.angle(v.x, v.y, v.heading.x, v.heading.y)
		v.r = (v.tutorial and canSee) and v.r or lerp(v.r, ang, 0.1)
		local vx, vy = math.cos(ang) * v.speed, math.sin(ang) * v.speed

		vx, vy = vx + v.v.x, vy + v.v.y
		moveWithPush(v, vx, vy, dt)

		-- Avoid getting stuck
		local _, _, _, len = world:check(v, v:xx() + vx * dt , v:yy() + vy * dt, function(item, other) return other.ttype == "enemy" and "slide" or nil end)
		if len > 0 then
			v.stuckTime = v.stuckTime + dt
		else
			v.stuckTime = 0
		end

		v.timingBetweenNodes = v.timingBetweenNodes + dt
		if canSee then
			if not v.tutorial then
				en.chase(v)
				en.alertOthers(v)
			else
				en.alertOthers(v)
			end
		end

		if math.sqrt((v.y - v.heading.y) ^ 2 + (v.x - v.heading.x) ^ 2) < 5 then
			v.pathIndex = v.pathIndex + 1
			v.timingBetweenNodes = 0

			if v.pathIndex <= #v.path then
				v.heading = v.path[v.pathIndex]
			else
				if v.loop == "reverse" then
					en.embark(v, reversePath(v.path), v.speed, v.loop)
				elseif v.loop == "circle" then
					en.embark(v, v.path, v.speed, v.loop)
				else
					v.heading = nil
					v.path = nil
					v.pathIndex = 1

					v.mode = "standingAround"
					v.speed = v.pace
					if not v.patrolAround then
						v.patrolAround = {x = math.floor(v.x / tileSize), y = math.floor(v.y / tileSize)}
					end
				end
			end
		end
		
		if v.timingBetweenNodes > 1 * (v.run / v.speed) then -- Been trying to get to next node for too long
			local p = generatePath(math.floor(v.x / tileSize), math.floor(v.y / tileSize), math.floor(v.path[#v.path].x / tileSize), math.floor(v.path[#v.path].y / tileSize))
			en.embark(v, p, v.speed, v.loop)
			v.timingBetweenNodes = 0
		end
	end

	if v.mode == "patrol" and not v.heading then
		v.mode = "standingAround"
		v.speed = v.pace
		if not v.patrolAround then
			v.patrolAround = {x = math.floor(v.x / tileSize), y = math.floor(v.y / tileSize)}
		end
	end

	if v.tutorial then
		if v.bt < v.btm then
			v.bt = v.bt + dt
		end

		if canSee then
			v.r = math.angle(v.x, v.y, player.x, player.y)
		end

		if canSee and v.bt >= v.btm then
			v.bt = 0
			local ang = v.r

			if v.weapon and v.weapon ~= "knife" and v.ammo > 0 then
				camera:shake(2, 0.02)

				if v.weapon == "pistol" then
					playSound("shoot")
				elseif v.weapon == "auto" then
					playSound("machinegun")
				elseif v.weapon == "shotgun" then
					playSound("shotgun")
				end

				if v.weapon == "shotgun" then
					for i = 0, 4 do
						table.insert(enBullets, {x = v.x, y = v.y, a = ang + math.rad(love.math.random(-15, 15)), v = 3000})
					end
				else
					table.insert(enBullets, {x = v.x, y = v.y, a = ang + (v.weapon == "auto" and math.rad(love.math.random(-4, 4)) or 0), v = 3000})
				end
			elseif v.weapon and v.ammo == 0 then
				v.bt = v.btm * 0.5
			end
		end
	end

	if v.mode == "chase" then
		if v.bt < v.btm then
			v.bt = v.bt + dt
		end

		if canSee and v.bt >= v.btm then
			v.bt = 0

			-- Shoot the messenger
			local ang = v.r

			if math.dist(v.x, v.y, player.x, player.y) <= v.h * 1.5 + player.h * 0.5 then
				playSound("slash")
				v.slash = flux.to(v, 0.1, {slashr = -math.rad(90) * 0.5}):ease("quadout"):oncomplete(function() v.slash = nil; v.slashr = math.rad(90) * 0.5 end)

				if math.abs(angleDifference(ang, math.angle(v.x, v.y, player.x, player.y))) < math.rad(90) * 0.5 then
					playSound("perished")

					killPlayer(player, ang)
					return
				end
			elseif v.weapon and v.weapon ~= "knife" and v.ammo > 0 then
				camera:shake(2, 0.02)

				if v.weapon == "pistol" then
					playSound("shoot")
				elseif v.weapon == "auto" then
					playSound("machinegun")
				elseif v.weapon == "shotgun" then
					playSound("shotgun")
				end

				if v.weapon == "shotgun" then
					for i = 0, 4 do
						table.insert(enBullets, {x = v.x, y = v.y, a = ang + math.rad(love.math.random(-15, 15)), v = 3000})
					end
				else
					table.insert(enBullets, {x = v.x, y = v.y, a = ang + (v.weapon == "auto" and math.rad(love.math.random(-4, 4)) or 0), v = 3000})
				end
				v.ammo = v.ammo - 1
			elseif v.weapon and v.ammo == 0 then
				v.bt = v.btm * 0.5
			end
		end
		
		
		v.timingBetweenNodes = v.timingBetweenNodes + dt

		local ang = math.angle(v.x, v.y, player.x, player.y)
		v.r = ang -- lerp?
		local vx, vy = math.cos(ang) * v.speed, math.sin(ang) * v.speed
		vx, vy = vx + v.v.x, vy + v.v.y
		moveWithPush(v, vx, vy, dt)

		if canSee then
			v.timingBetweenNodes = 0
		end

		if not canSee and v.timingBetweenNodes > 1 then
			v.mode = "patrol"
			v.timingBetweenNodes = 0

			local p = generatePath(math.floor(v.x / tileSize), math.floor(v.y / tileSize), math.floor(player.x / tileSize), math.floor(player.y / tileSize))
			en.embark(v, p, v.run)

			flux.to(v, 0.2, {alertScale = 0}):ease("quadout")
		end
	end
end

local debug = false
function en.draw(v)
	v.scaleArrow = math.clamp(1, v.scaleArrow + love.timer.getDelta() * v.signArrow * 0.5, 1.25)

	if v.scaleArrow <= 1 or v.scaleArrow >= 1.25 then
		v.signArrow = -v.signArrow
	end

	if v.tutorial then
		love.graphics.draw(automataBase, v.x, v.y, v.r, 1, 1, v.w * 0.5, v.h * 0.5)

		love.graphics.setColor(255 * (v.scaleArrow - 1) * 4, 0, 0)
		love.graphics.draw(automataBlinker, v.x, v.y, 0, 1, 1, automataBlinker:getWidth() * 0.5, automataBlinker:getHeight() * 0.5)
	else
		love.graphics.setColor(HSL(v.color, 255, 128))
		love.graphics.draw(v.img, v.x, v.y, v.r, 1, 1, v.w * 0.5, v.h * 0.5)
	end

	love.graphics.setColor(255, 255, 255)
	if v.stunned > 0 then
		love.graphics.draw(stunImg, v.x, v.y, math.pi * 2 * v.stunned, 1, 1, stunImg:getWidth() * 0.5, stunImg:getHeight() * 0.5)
	end

	if v.weapon and v.weapon ~= "knife" then
		local img = guns[v.weapon][5]
		love.graphics.draw(img, v.x, v.y, v.r + math.pi * 0.5, 1, 1, img:getWidth() * 0.5, img:getHeight() * v.scaleArrow + v.img:getHeight() * 0.5)
	else
		love.graphics.draw(pointerImg, v.x, v.y, v.r + math.pi * 0.5, v.scaleArrow, v.scaleArrow, pointerImg:getWidth() * 0.5, pointerImg:getHeight() * 3)
	end

	love.graphics.draw(alertImg, v.x, v.y, 0, v.alertScale, v.alertScale, alertImg:getWidth() * 0.5, alertImg:getHeight())

	if v.slash then
		love.graphics.draw(guns["knife"], v.x + math.cos(v.slashr + v.r) * v.w, v.y + math.sin(v.slashr + v.r) * v.h, v.r + v.slashr + math.pi * 0.5, 1, 1, bulletImg:getWidth() * 0.5, bulletImg:getHeight() * 0.5)
	end

	if debug then
		love.graphics.circle("line", v.x, v.y, 500)
		love.graphics.arc("line", v.x, v.y, 300, v.r - v.fov * 0.5, v.r + v.fov * 0.5)
	end
end

function move(obj, dx, dy, dt)
	local ax, ay, cols = world:move(obj, obj:xx() + dx * dt, obj:yy() + dy * dt, function(item, other)
		if (other.type == "prop" or item.type == "prop") or (other.ttype == "enemy" and item.ttype == "enemy" and (item.patrolAround or other.patrolAround or item.stuckTime >= 0.5 or other.stuckTime >= 0.5)) then
			return nil
		else
			return "slide"
		end
	end)
	obj.x = ax + obj.w * 0.5
	obj.y = ay + obj.h * 0.5

	return cols
end

function moveProp(obj, dx, dy, dt)
	local ax, ay, cols = world:move(obj, obj:xx() + dx * dt, obj:yy() + dy * dt, function(item, other)
		if (other.hueCycle and other.type == "push") then
			return "slide"
		end

		if (other.type == "push" or other.type == "prop") then
			return nil
		else
			return "slide"
		end
	end)
	obj.x = ax + obj.w * 0.5
	obj.y = ay + obj.h * 0.5

	return cols
end

function moveWithPush(obj, dx, dy, dt)
	local cols = move(obj, dx, dy, dt)

	for i,m in ipairs(cols) do
		if (m.item.ttype == "enemy" and m.other.ttype == "player") or (m.item.ttype == "player" and m.other.ttype == "enemy") then
			local v = m.item.ttype == "enemy" and m.item or m.other

			if not v.tutorial then
				v.mode = "chase"
				v.speed = v.run
				v.timingBetweenNodes = 0
				v.patrolAround = nil
			end
		end

		if not m.other.tutorial and ((m.other.type == "push" or m.other.type == "prop") and m.other.ttype ~= "immovable") then
			move(m.other, -m.normal.x * obj.speed, -m.normal.y * obj.speed, dt)
		end
	end
end

function table.clone(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end