require "map"

-- Own Libraries
require "lib/camera"
require "lib/fps"

-- External Libraries
require "lib/generalMath"
require "lib/sound"
flux = require "lib/flux"
bump = require "lib/bump"
shine = require "lib/shine"
push = require "lib/push"

require "enemy"
require "misc"
require "color"
require "shoot"
require "guns"
require "ability"
require "particle"
require "textbox"

love.graphics.setDefaultFilter("linear", "linear")
love.graphics.setLineStyle("rough")
love.graphics.setLineWidth(2)

w, h = 1280, 720 -- 720p in-game resolution

push:setupScreen(w, h, w, h, { fullscreen = false, resizable = false, highdpi = true, canvas = true })

map = {}
mapW, mapH = nil, nil
tileSize = 32

world = bump.newWorld(32)

back = {
	c = love.graphics.newCanvas(w * 1.5, h * 1.5),
	r = 0,
}

pause = false
pauseText = "PAUSED"

totalEnemies = 0

GLOBAL_STATE = "Rude Awakening"

CURRENT_MAP = "map/Warehouse/--"

function love.load()
	newSound("sfx/shoot.ogg")
	newSound("sfx/machinegun.ogg")
	newSound("sfx/shotgun.ogg")

	newSound("sfx/explode.ogg")
	newSound("sfx/slash.ogg")
	newSound("sfx/perished.ogg")
	newSound("sfx/glassbreak.ogg")
	newSound("sfx/talk.ogg")
	newSound("sfx/talkDeep.ogg")

	newSound("sfx/select.ogg")
	newSound("sfx/lightbulb.ogg", true)
	newSound("sfx/rain.ogg", true)

	newSound("sfx/holdTeleport.ogg", true)
	newSound("sfx/teleport.ogg")
	newSound("sfx/shield.ogg")

	newSound("sfx/openLock.ogg")
	newSound("sfx/doorClose.ogg")

	-- Music --
	newSound("music/mainTheme.ogg", true)
	newSound("music/fightTheme.ogg", true)

	local cur = love.mouse.newCursor("img/cursor.png", 8, 8)
	curImg = love.graphics.newImage("img/cursor.png")
	pathImg = love.graphics.newImage("img/misc/path.png")
	love.mouse.setCursor(cur)
	love.mouse.setGrabbed(false)

	-- Menu images --------------------------------------------------------
	menu = {
		building 	= love.graphics.newImage("img/menu/building.png"),
		lamp 		= love.graphics.newImage("img/menu/lamp.png"),
		heart 		= love.graphics.newImage("img/menu/heartcolour.png"),
		rain 		= love.graphics.newImage("img/menu/rain.png"),

		a = 0,
		c = color.violet,

		panCam = 0,

		max = 5,
		cur = 0,

		state = "main",
	}
	menu.rain:setWrap('repeat','repeat')

	menu.mesh1 = newRainMesh()
	menu.mesh2 = newRainMesh()
	menu.mesh3 = newRainMesh()

	lightbulbFlicker()
	panCamera()
	-- Fancy shaders ------------------------------------------------------
	shaders = {
		split = shine.separate_chroma(),
		
		grain = shine.filmgrain(),
		glow = shine.boxblur(),

		fullblur = shine.gaussianblur(),
	}
	shaders.split.radius = 0
	shaders.fullblur.sigma = 6
	shaders.glow.radius = 0 -- 0 to 4
	shaders.grain.opacity = 0 -- 0 to 1

	shaders.full = shaders.split:chain(shaders.glow):chain(shaders.grain)
	-----------------------------------------------------------------------

	local heart = love.graphics.newImage("img/hearts.png")
	heart:setWrap("repeat", "repeat")
	local quad = love.graphics.newQuad(0, 0, w * 1.5, h * 1.5, heart:getWidth(), heart:getHeight())
	
	love.graphics.setCanvas(back.c)
	--love.graphics.clear(0, 0, 0, 255)
	--stripes()
	love.graphics.draw(heart, quad)
	love.graphics.setCanvas()

	player = {
		x = 16,
		y = 16,
		w = 24,
		h = 24,

		xx = function(self)
			return self.x - self.w * 0.5
		end,
		yy = function(self)
			return self.y - self.h * 0.5
		end,

		speed = 200,
		img = love.graphics.newImage("img/player.png"),
		r = 0,
		type = "push",
		ttype = "player",

		bullets = {},

		slash = nil,
		slashr = math.rad(150) * 0.5,

		inv = {
			[1] = {
				weapon = nil,
			}, -- corresponds to weapon, usually
			[2] = {
				grenade = "explode",
				ammo = 0,
			},
			[3] = {
				grenade = "noise",
				ammo = 0,
			},
			[4] = {
				grenade = "stun",
				ammo = 0,
			},
		},
		abilityInv = {
			{"spatial dilation", nil},
			{"psyche wall", nil},
		},
		selected = 1,
		abilitySelected = 1,

		stunned = 0,

		abilityDrain = 0, -- 0 to 1

		notMoveTime = 0, -- if more than 0, cannot move

		heart = {
			img = love.graphics.newImage("img/misc/heart.png"),
			c = love.math.random(0, 255),
		},

		overlayOp = 0,
	}
	bulletImg = love.graphics.newImage("img/misc/bullet.png")
	grenadeImg = love.graphics.newImage("img/misc/grenade.png")
	--world:add(player, player:xx(), player:yy(), player.w, player.h)

	--restart()

	tweenRotA(2)
	tweenRotAA(2)
	
	deadSign = love.graphics.newImage("img/signs/deadSign.png")
	pausedSign = love.graphics.newImage("img/signs/pausedSign.png")
	unconSign = love.graphics.newImage("img/signs/unconsciousSign.png")
	deadTimer = 0

	font = love.graphics.newFont("img/visitor.ttf", 32)
	love.graphics.setFont(font)

	RudeAwakeningString = [[Protocol 3 proudly presents]]

	-- Brooklyn, 1930; The Docks
	triggers.typeOutText = true


	settings = {
		updateEnemiesWhileDead = false,
		updateBulletsWhileDead = false,
	
		abilitiesButton = 2,
		meleeButton = 3,
		mainButton = 1,

		weaponInv = "lshift",
		abilityInv = "lctrl",
		changeSlot = "mouseWheel",
		pickUp = "e",
	}
end


function love.update(dt)
	flux.update(dt)

	dt = dt * triggers.dtMult

if GLOBAL_STATE == "game" then
	updateText(dt)
	updateExplosion(dt)
	updateProps(dt)
	camera:update(dt)

	if deadTimer > 0 then
		deadTimer = deadTimer - dt
	end

	if settings.updateEnemiesWhileDead and ((pause and pauseText == "DEAD") or not pause) then
		for i,v in ipairs(enemies) do
			en.update(v, dt)
		end
	end

	if settings.updateBulletsWhileDead then
		updateBullets(dt)
	end

if not pause then
	updateParticles(dt)

	if not settings.updateEnemiesWhileDead then
		for i,v in ipairs(enemies) do
			en.update(v, dt)
		end
	end

	if pause then return end -- The player got slashed

	local vx, vy = 0, 0
	local mlt = 1

	player.stunned = math.max(0, player.stunned - dt)
	player.heart.c = (player.heart.c + dt * 255 * 0.25) % 256

	if love.keyboard.isDown("w") then
		vy = vy - player.speed * (player.stunned > 0 and -1 or 1)
	end
	if love.keyboard.isDown("s") then
		vy = vy + player.speed * (player.stunned > 0 and -1 or 1)
	end
	if love.keyboard.isDown("a") then
		vx = vx - player.speed * (player.stunned > 0 and -1 or 1)
	end
	if love.keyboard.isDown("d") then
		vx = vx + player.speed * (player.stunned > 0 and -1 or 1)
	end

	if math.abs(vx) > 0 and math.abs(vy) > 0 then
		mlt = 1 / math.sqrt(2)
	end

	if player.notMoveTime == 0 then
		moveWithPush(player, vx * mlt, vy * mlt, dt)
	end

	if CURRENT_MAP == "map/Warehouse/Tutorial" then
		local ctx, cty = math.floor(player.x/tileSize), math.floor(player.y/tileSize)
		if ctx == 6 and cty == 13 and not triggers.r1 then
			pause = "cutscene"
			displayText(professor, {"...", "Oh, don't worry. They're machines. Just machines..."})
			triggers.r1 = true
		elseif ((ctx == 5 and cty == 8) or (ctx == 5 and cty == 9)) and not triggers.r2 then
			pause = "cutscene"
			displayText(nil, {"Hold shift to see your inventory. Scroll through it using the mousewheel.", "You just have some explosive grenades - equip one and use left-click to bounce the grenade into the room."})
			triggers.r2 = true
		end
	end

	local mx, my = mapW * tileSize * 0.5, mapH * tileSize * 0.5
	local mouseX, mouseY = push:mousePosition()
	mouseX, mouseY = mouseX - w * 0.5, mouseY - h * 0.5
	camera:lookAt((player.x + mx) * 0.5 + mouseX * 0.125, (player.y + my) * 0.5 + mouseY * 0.125)

	player.r = math.angle(player.x, player.y, camera:mousePosition())

	local s = player.inv[player.selected]
	if love.mouse.isDown(1) and s.weapon then
		s.bt = s.bt + dt
		if s.bt >= s.btm and player.notMoveTime == 0 then
			shoot()
			s.bt = 0

			if CURRENT_MAP == "map/Warehouse/Tutorial" and displayedProfessor and math.dist(player.x, player.y, 5.5 * tileSize, 14.5 * tileSize) > w / 3 then
				for i,v in ipairs(props) do
					if v.ttype == "proff" then table.remove(props, i); break end
				end
			end
		end
	else
		s.bt = s.btm
	end
	
	if not settings.updateBulletsWhileDead then updateBullets(dt) end
	for i,v in ipairs(player.bullets) do
		local ox, oy = v.x, v.y
		
		if v.type ~= "prop" then
			v.x = v.x + math.cos(v.a) * v.v * dt
			v.y = v.y + math.sin(v.a) * v.v * dt

			if checkMap(math.floor(v.x / tileSize), math.floor(v.y / tileSize)) ~= 0 then
				splatter(v.x, v.y, 20, 40)
				table.remove(player.bullets, i)
			else -- and ((item.type == "push" and item.ttype ~= "furniture"); (item ~= player) 
				local items = world:querySegment(ox, oy, v.x, v.y, function(item) return item.ttype == "enemy" or item.type == "wall" or (item.hueCycle and item.type == "push") end)
				--local items = world:queryPoint(v.x, v.y, function(item) return item ~= player end)
				if #items > 0 then
					table.remove(player.bullets, i)

					--playSound("perished")
					if items[1].ttype == "enemy" then en.die(items[1], v.a, true) end
					if (items[1].hueCycle and items[1].type == "push") then
						playSound("glassbreak")
						removeElement(props, items[1])
						world:remove(items[1])
					end
				end
			end
		else
			-- Move grenades
			local ax, ay, cols = world:move(v, v.x + v.vx * v.v * dt, v.y + v.vy * v.v * dt, function(item, other) return other.type == "wall" and "bounce" or nil end)
			v.x, v.y = ax, ay

			if #cols >= 1 then
				local norm = cols[1].normal
				if norm.x ~= 0 then
					v.vx = -v.vx
				end
				if norm.y ~= 0 then
					v.vy = -v.vy
				end
			end

			-- TIME TO BLOW
			if v.v <= 0 then
				addExplosion(v.x, v.y, v.blowRad)
				playSound("explode")
				if v.grenadeType ~= "stun" then en.canHear(enemies, v.x, v.y, v.blowRad * 2) end

				if v.grenadeType == "explode" then splatter(v.x, v.y, 30, 50) end
				local absoluteRadius = v.grenadeType == "noise" and v.blowRad or v.blowRad * 0.8
				-- Weird workaround - otherwise, it will not loop over some enemies after people have died
				local remTbl = {}
				for _,l in ipairs(enemies) do
					local dist = math.dist(l.x, l.y, v.x, v.y)

					if (dist <= absoluteRadius) or (en.canSeeExplosion(l, v.x, v.y) and dist <= v.blowRad) then
						if v.grenadeType == "explode" then
							en.die(l, math.angle(v.x, v.y, l.x, l.y))
							table.insert(remTbl, l)
						elseif v.grenadeType == "stun" then
							en.stun(l)
						end
					end
				end
				for _,l in ipairs(remTbl) do
					removeElement(enemies, l)
				end

				if math.dist(player.x, player.y, v.x, v.y) <= absoluteRadius then
					if v.grenadeType == "explode" then
						killPlayer(player, math.angle(v.x, v.y, player.x, player.y))
					elseif v.grenadeType == "stun" then
						player.stunned = 5
					end
				end
				table.remove(player.bullets, i)
			end
		end

		if #enemies == 0 and not triggers.r3 and displayedProfessor then
			world:remove(wallTable[#wallTable]); table.remove(wallTable, #wallTable)
			triggers.r3 = true
		end	
	end

	if #enemies == totalEnemies - 1 and triggers.dtMult == 1 and not triggers.r5 and CURRENT_MAP == "map/Warehouse/2" then
		pause = "cutscene"
		displayText(gramophone, {
			"It seems that my \"gift\" was not wasted on you. I am quite impressed.",
			"Your brain should adjust to your new perception after some time.",
			"Until then, I would not advise you to use your \"gift\" too much. The consequences for you could be... dire.",
			"Now, why don't you practice a bit? We can talk more when you get out."
			})

		triggers.r5 = true
	end

	-- Ability Drain on the player
	if love.mouse.isDown(settings.abilitiesButton) and player.abilityInv[player.abilitySelected] then
		if player.abilityInv[player.abilitySelected][1] ~= "spatial dilation" then
			player.abilityDrain = math.min(1, player.abilityDrain + dt * 0.5) -- you can hold it for 2 seconds
		end
	else
		player.abilityDrain = math.max(0, player.abilityDrain - dt * 0.4) -- recharges in 2.5 seconds
	end

	if player.abilityDrain == 1 then
		killPlayer(player, 0, 0)
	end

	shaders.glow.radius = player.abilityDrain * 8 -- 0 to 4
	shaders.grain.opacity = player.abilityDrain -- 0 to 1
end
elseif GLOBAL_STATE == "menu" then
	local u, v
	for k = 1,3 do
		for i = 1,4 do
			u, v = menu["mesh"..k]:getVertexAttribute(i, 2)
			u, v = (u + dt * (k/12)), (v - dt * 0.5)
			menu["mesh"..k]:setVertexAttribute(i, 2, u, v)
		end
	end
elseif GLOBAL_STATE == "Rude Awakening" then
	updateText(dt)
end

	--love.window.setTitle(love.timer.getFPS())
end

function love.draw()
	push:start()
if GLOBAL_STATE == "game" then
	love.graphics.setColor(255, 255, 255)
	shaders.full(function()
		local parallaxStrength = 0.3
		local mx, my = mapW * tileSize * 0.5, mapH * tileSize * 0.5
		mx, my = camera:worldToScreen(mx, my)
		local px, py = camera:worldToScreen(player.x, player.y)

		love.graphics.draw(back.c, mx + (px - mx) * parallaxStrength, my + (py - my) * parallaxStrength, back.r, 1, 1, back.c:getWidth() * 0.5, back.c:getHeight() * 0.5)

		camera:set()
			love.graphics.setColor(0, 0, 0)
			for y = 0, mapH - 1 do
				for x = 0, mapW - 1 do
					if checkMap(x, y) == 0 then
						love.graphics.rectangle("fill", x * tileSize, y * tileSize, tileSize, tileSize)
					end
				end
			end

			drawProps(true)
			for i,v in ipairs(wallTable) do
				love.graphics.setColor(0, 0, 0)
				love.graphics.rectangle("fill", v.x, v.y, v.w, v.h)
				love.graphics.setColor(255, 255, 255)
				love.graphics.rectangle("line", v.x, v.y, v.w, v.h)
			end

			love.graphics.setColor(255, 255, 255)
			for y = 0, mapH - 1 do
				for x = 0, mapW - 1 do
					if checkMap(x, y) == 0 then
						if checkMap(x - 1, y) ~= 0 then
							love.graphics.line(x * tileSize, y * tileSize, x * tileSize, (y + 1) * tileSize)
						end

						if checkMap(x + 1, y) ~= 0 then
							love.graphics.line((x + 1) * tileSize, y * tileSize, (x + 1) * tileSize, (y + 1) * tileSize)
						end

						if checkMap(x, y - 1) ~= 0 then
							love.graphics.line(x * tileSize, y * tileSize, (x + 1) * tileSize, y * tileSize)
						end

						if checkMap(x, y + 1) ~= 0 then
							love.graphics.line(x * tileSize, (y + 1) * tileSize, (x + 1) * tileSize, (y + 1) * tileSize)
						end
					end
				end
			end

			for i,v in ipairs(pathList) do
				for i = 2, #v do
					local f, b = v[i], v[i-1]
					love.graphics.draw(pathImg, (b.x + f.x) * 0.5, (b.y + f.y) * 0.5 , math.atan2(f.y - b.y, f.x - b.x), 1, 1, pathImg:getWidth() * 0.5, pathImg:getHeight() * 0.5)
				end
			end
			
			drawBullets()

			for i,v in ipairs(enemies) do
				en.draw(v)
			end

			if pauseText ~= "DEAD" then
				love.graphics.draw(player.img, player.x, player.y, player.r, 1, 1, player.w * 0.5, player.h * 0.5)

				love.graphics.setColor(HSL(player.heart.c, 255, 128))
				love.graphics.draw(player.heart.img, player.x, player.y, 0, 1, 1, player.heart.img:getWidth() * 0.5, player.heart.img:getHeight() * 0.5)

				love.graphics.setColor(255, 255, 255)
				if player.stunned > 0 then
					love.graphics.draw(stunImg, player.x, player.y, math.pi * 2 * player.stunned, 1, 1, stunImg:getWidth() * 0.5, stunImg:getHeight() * 0.5)
				end

				local s = player.inv[player.selected]
				if s.weapon then
					local img = guns[s.weapon][5]
					love.graphics.draw(img, player.x, player.y, player.r + math.pi * 0.5, 1, 1, img:getWidth() * 0.5, img:getHeight() + player.img:getHeight() * 0.5)
				end

				if player.slash then
					love.graphics.draw(guns["knife"], player.x + math.cos(player.slashr + player.r) * player.w, player.y + math.sin(player.slashr + player.r) * player.h, player.r + player.slashr + math.pi * 0.5, 1, 1, bulletImg:getWidth() * 0.5, bulletImg:getHeight() * 0.5)
				end
			end

			for i,v in ipairs(player.bullets) do
				love.graphics.draw(v.type == "prop" and grenadeImg or bulletImg, v.x, v.y, v.type == "prop" and math.tan(v.vy/v.vx) or (v.a + math.pi * 0.5), 1, 1, bulletImg:getWidth() * 0.5, bulletImg:getHeight() * 0.5)
			end

			if love.mouse.isDown(settings.abilitiesButton) and not pause then
				if not player.abilityInv[player.abilitySelected] then goto skipAbility end
				
				local s = player.abilityInv[player.abilitySelected][1]
				local x, y = camera:mousePosition()
				if s == "spatial dilation" then
					local mx, my = x - player.w * 0.5, y - player.h * 0.5
					local _, len = world:queryRect(mx, my, player.w, player.h, function(item) return item.type == "wall" or item.type == "push" end)

					local allGood = (len == 0 and canUseAbility(s))

					love.graphics.setColor(255, 255, 255)
					love.graphics.setLineWidth(love.math.random(0, 4))
					love.graphics.circle("line", player.x, player.y, 150)
					if allGood then
						love.graphics.line(player.x, player.y, x, y)

						love.graphics.setLineWidth(2)
					else
						love.graphics.setLineWidth(1)
					end
					
					love.graphics.setColor(255, 255, 255)
					love.graphics.rectangle("line", mx, my, player.w, player.h)
					love.graphics.setLineWidth(2)

					local isAtSomeone = false
					for i,v in ipairs(enemies) do
						if x >= v:xx() and x <= v:xx() + v.w and y >= v:yy() and en.canBeSeen(v) and y <= v:yy() + v.h then
							player.abilityInv[player.abilitySelected][2] = v
							isAtSomeone = true
							break
						end
					end

					if not isAtSomeone and canUseAbility(s) then
						player.abilityInv[player.abilitySelected][2] = nil
					end

					if player.abilityInv[player.abilitySelected][2] then
						local v = player.abilityInv[player.abilitySelected][2]
						love.graphics.setColor(255, 255, 255)
						love.graphics.draw(eyeImg, v.x, v.y, 0, 1, 1, eyeImg:getWidth() * 0.5, eyeImg:getHeight() * 0.5)
					end
				elseif s == "psyche wall" then
					love.graphics.setLineWidth(love.math.random(0, 4))
					love.graphics.circle("line", player.x, player.y, 28 )
					love.graphics.setLineWidth(2)
				end

				::skipAbility::
			end

			drawParticles()
			drawProps(false)

			-- Draw inventory
			if love.keyboard.isDown(settings.weaponInv) then
				for i=1,4 do
					local ang = (i-2) * math.pi * 0.5
					local r = 2 * player.w
					local rr = 16

					love.graphics.setColor(0, 0, 0)
					love.graphics.circle("fill", player.x + math.cos(ang) * r, player.y + math.sin(ang) * r, rr)
					
					if i == player.selected then
						love.graphics.setColor(255, 0, 0)
					else
						love.graphics.setColor(255, 255, 255)
					end
					love.graphics.circle("line", player.x + math.cos(ang) * r, player.y + math.sin(ang) * r, rr)
					
					love.graphics.setColor(255, 255, 255)
					if player.inv[i].weapon then
						local img = guns[player.inv[i].weapon][1]
						love.graphics.draw(img, player.x + math.cos(ang) * r, player.y + math.sin(ang) * r, math.pi * 0.5, 1, 1, img:getWidth() * 0.5, img:getHeight() * 0.5)
					elseif player.inv[i].grenade then
						local img = grenades[player.inv[i].grenade][1]
						love.graphics.draw(img, player.x + math.cos(ang) * r, player.y + math.sin(ang) * r, 0, 1, 1, img:getWidth() * 0.5, img:getHeight() * 0.5)
					end
				end
			end

			drawExplosion()
		camera:unset()
end)
	if pause then
		if pauseText == "DEAD" then
			if CURRENT_MAP == "map/Warehouse/Tutorial" then
				love.graphics.draw(unconSign, w * 0.5, h * 0.5, camera.rotation, 1, 1, unconSign:getWidth() * 0.5, unconSign:getHeight() * 0.5)
			else
				love.graphics.draw(deadSign, w * 0.5, h * 0.5, camera.rotation, 1, 1, deadSign:getWidth() * 0.5, deadSign:getHeight() * 0.5)
			end
		elseif pause == "cutscene" then
			drawText()
		else
			love.graphics.draw(pausedSign, w * 0.5, h * 0.5, camera.rotation, 1, 1, pausedSign:getWidth() * 0.5, pausedSign:getHeight() * 0.5)
		end
	end

	local abilityNames = {
		"spatial displacement",
		"psyche wall"
	}

	printText(string.format("%1.f. %s - %1.f", player.selected, player.inv[player.selected].weapon or player.inv[player.selected].grenade or "nothing", player.inv[player.selected].ammo or 0), 0, 0)
	printText(string.format("%s", player.abilityInv[player.abilitySelected] and abilityNames[player.abilitySelected] or "N/A"), 0, font:getHeight())
	printText(string.format("%1.f / %1.f Enemies", #enemies, totalEnemies), 0, font:getHeight() * 2)
elseif GLOBAL_STATE == "menu" then
	love.graphics.setColor(255, 255, 255)

	love.graphics.draw(menu.mesh1, 0, 0, 0, w/menu.rain:getWidth(), h/menu.rain:getHeight())

	--shaders.fullblur(function()
		love.graphics.draw(menu.building, 0, -13 + menu.panCam) -- the 13 is there to compensate for added height on edited img
	--end)

	love.graphics.draw(menu.mesh2, -10, 0, 0, w/menu.rain:getWidth(), h/menu.rain:getHeight())

	love.graphics.draw(menu.lamp, 108, 3)

	love.graphics.draw(menu.mesh3, 10, 0, 0, w/menu.rain:getWidth(), h/menu.rain:getHeight())

	love.graphics.setBlendMode("add")
	love.graphics.setColor(menu.c[1], menu.c[2], menu.c[3], menu.a)

	lightbulbVol(menu.a / 255)

	love.graphics.draw(menu.heart)
	love.graphics.setBlendMode("alpha")

	local str = "YGD Build"
	love.graphics.setColor(0, 0, 0)
	love.graphics.print(str, w - font:getWidth(str), h - font:getHeight())
	love.graphics.setColor(255, 255, 255)
	love.graphics.print(str, w - font:getWidth(str), h - font:getHeight() - 1)

	local x, y = push:mousePosition()
	local ss = push._SCALE.x
	if x >= 525*ss and x <= (525 + font:getWidth(">Rude Awakening"))*ss
		and y >= 316*ss and y <= (316 + (menu.max) * font:getHeight())*ss then
			local new =  math.max(0, math.min(menu.max - 1, math.floor((y - 316*ss) / (font:getHeight()*ss))))
			if menu.cur ~= new and (menu.state == "level" or menu.state == "main") then playSound("select") end
			menu.cur = new
	end 

	if menu.state == "main" then
		love.graphics.setColor(0, 0, 0)
		love.graphics.print("Start\nLevel Editor\nControls\nCredits\nQuit", font:getWidth(">") + 525, 316)
		love.graphics.print(">", 525, 316 + menu.cur * font:getHeight())

		love.graphics.setColor(255, 255, 255)
		love.graphics.print("Start\nLevel Editor\nControls\nCredits\nQuit", font:getWidth(">") + 525, 315)
		love.graphics.print(">", 525, 315 + menu.cur * font:getHeight())
	elseif menu.state == "level" then
		love.graphics.setColor(0, 0, 0)
		love.graphics.print("Tutorial\nRude Awakening\nThe Agreement\nBack", font:getWidth(">") + 525, 316)
		love.graphics.print(">", 525, 316 + menu.cur * font:getHeight())

		love.graphics.setColor(255, 255, 255)
		love.graphics.print("Tutorial\nRude Awakening\nThe Agreement\nBack", font:getWidth(">") + 525, 315)
		love.graphics.print(">", 525, 315 + menu.cur * font:getHeight())
	elseif menu.state == "options" then
		local str = [[
Controls:
	WASD        - move
	Space       - melee
	E           - pick-up/throw weapon
	Control     - switch abilities
	Mouse Wheel - change equiped weapon

	Shift        - view inventory
	Left-Click   - shoot
	Right-Click  - ability
	Middle-Click - melee

	F11 - fullscreen toggle
	Tab - pause game
	Tab + Escape - go back to menu from level
		]]

		love.graphics.setColor(0, 0, 0)
		love.graphics.print(str, font:getWidth(">") + 525, 316)

		love.graphics.setColor(255, 255, 255)
		love.graphics.print(str, font:getWidth(">") + 525, 315)
	elseif menu.state == "credits" then
		local str = [[
Programming and Art
	David Khachaturov
Story and Level Design
	Avanes Khachaturov
Music and Creative
	Zacharie Sciamma
		]]

		love.graphics.setColor(0, 0, 0)
		love.graphics.print(str, font:getWidth(">") + 525, 316)

		love.graphics.setColor(255, 255, 255)
		love.graphics.print(str, font:getWidth(">") + 525, 315)
	end
elseif GLOBAL_STATE == "Rude Awakening" then
	love.graphics.setColor(255, 255, 255)
	love.graphics.rectangle("fill", 0, (h - triggers.eyelidHeight) * 0.5, w, triggers.eyelidHeight)

	if triggers.typeOutText then
		triggers.typeOutTimer = math.min(#RudeAwakeningString, triggers.typeOutTimer + love.timer.getDelta() * 12)
		local ss = string.sub(RudeAwakeningString, 1, math.floor(triggers.typeOutTimer))
		if ss ~= triggers.typeOutString then
			playSound("talk", true)
		end

		triggers.typeOutString = ss
	end

	local std = triggers.typeOutText and triggers.typeOutString or RudeAwakeningString

	if RudeAwakeningString == "Rude Awakening" then love.graphics.setColor(0, 0, 0) end
	love.graphics.printf(std, 0, (h - font:getHeight(std)) * 0.5, w, "center")--(w - font:getWidth(std)) * 0.5, (h - font:getHeight(std)) * 0.5)

	if std == RudeAwakeningString and triggers.typeOutText and CURRENT_MAP == "map/Warehouse/Tutorial" then
		triggers.typeOutTimer = 1
		triggers.typeOutText = false

		flux.to(player, 1, {overlayOp = 255}):delay(2):ease("linear"):oncomplete(function() RudeAwakeningString = ""; flux.to(player, 0.1, {overlayOp = 0}) end):after(triggers, 4, {eyelidHeight = 128}):delay(2):after(0.5, {eyelidHeight = 0}):oncomplete(function() RudeAwakeningString = "Rude Awakening" end):after(3, {eyelidHeight = h}):delay(0.5):ease("cubicout"):after(player, 1, {overlayOp = 255}):ease("linear"):oncomplete(function() GLOBAL_STATE = "game"; CURRENT_MAP = "map/Warehouse/2"; restart(); playBackgroundMusic("fightTheme") end):after(0.5, {overlayOp = 0}):ease("linear")
	end

	if std == RudeAwakeningString and triggers.typeOutText and RudeAwakeningString == "Protocol 3 proudly presents" then
		triggers.typeOutTimer = 1
		triggers.typeOutText = false
		flux.to(player, 1, {overlayOp = 255}):delay(2):ease("linear"):oncomplete(function() RudeAwakeningString = "Recalling Monochrome" end):after(2, {overlayOp = 0}):ease("linear"):delay(2):after(1, {overlayOp = 255}):delay(3):ease("linear"):oncomplete(function() goBackToMenu() end):after(3, {overlayOp = 0}):ease("quadin")
	end

	if std == RudeAwakeningString and triggers.typeOutText and string.find(RudeAwakeningString, "Now then.") then
		triggers.typeOutTimer = 1
		triggers.typeOutText = false

		flux.to(player, 1, {overlayOp = player.overlayOp}):oncomplete(function()
		RudeAwakeningString = [[You will find them in the study.
Acquire them at all cost. I care not of the consequences...
As long as you fulfill the agreement.]]

		triggers.typeOutText = true
		end)
	end

	if std == RudeAwakeningString and triggers.typeOutText and string.find(RudeAwakeningString, "You will find them in the study.") then
		triggers.typeOutTimer = 1
		triggers.typeOutText = false

		flux.to(player, 1, {overlayOp = 255}):delay(2):ease("linear"):oncomplete(function() RudeAwakeningString = "Upstate New York, 1930" end):after(2, {overlayOp = 0}):ease("linear"):delay(2):after(1, {overlayOp = 255}):delay(3):ease("linear"):oncomplete(function() RudeAwakeningString = "The Agreement" end):after(2, {overlayOp = 0}):ease("linear"):delay(2):after(1, {overlayOp = 255}):delay(3):oncomplete(function() GLOBAL_STATE = "game"; CURRENT_MAP = "map/Warehouse/3"; restart(); playBackgroundMusic("fightTheme") end):after(1, {overlayOp = 0}):ease("linear")
	end

	if pause == "cutscene" then
		drawText()
	end
end
	push:finish()

	love.graphics.setColor(0, 0, 0, player.overlayOp)
	love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())

	love.graphics.setColor(255, 255, 255)
	local mx, my = push:mousePosition()
	if GLOBAL_STATE ~= "Rude Awakening" then
		love.graphics.draw(curImg, mx, my, 0, 1, 1, 8, 8)
	end
end

function love.mousepressed(x, y, button)
if GLOBAL_STATE == "game" then
	if not pause then
		if button == settings.mainButton then
			if player.inv[player.selected].grenade then
				shoot()
			elseif player.selected == 1 and not player.inv[1].weapon then
				slash()
			end
		end

		if button == settings.meleeButton then
			slash()
		end

		if button == settings.abilitiesButton then
			ability("p")
		end
	end

	if pause == "cutscene" then
		keyText(button)
	end
elseif GLOBAL_STATE == "menu" then
	--if x >= 525 and x <= 525 + font:getWidth(">Level Editor")
		--and y >= 316 and y <= 316 + (menu.max) * font:getHeight() then
			activateMenu()
	--end
elseif GLOBAL_STATE == "Rude Awakening" then
	if pause == "cutscene" then
		keyText(button)
	end
end
end

function love.mousereleased(x, y, button)
if GLOBAL_STATE == "game" then
	if not pause then
		if button == settings.abilitiesButton then
			ability("r")
		end
	end
elseif GLOBAL_STATE == "menu" then
end
end

function love.keypressed(key)
	if key == "f11" then
		push:switchFullscreen()
	end

if GLOBAL_STATE == "game" then
	if pauseText ~= "DEAD" then
		if pause and key == "r" then
			restart()
		elseif pause == true and key == "escape" then
			goBackToMenu()
		end

		if key == "tab" and pause ~= "cutscene" then
			pause = not pause
			love.mouse.setGrabbed(not pause)
		elseif pause then
			if key:match("%l") then
				if pause == "cutscene" then
					keyText(key)
					if not pause then
						return
					end
				else
					pause = false
					love.mouse.setGrabbed(not pause)
				end
			end
		end
	end

	if pause and pauseText == "DEAD" and deadTimer <= 0 then
		if key:match("%l") then
			pauseText = "PAUSED"
			pause = false
			restart()
		end
	end

	if not pause then
		if key == "space" then
			slash()
		end

		if key == settings.abilityInv and not love.mouse.isDown(settings.abilitiesButton) then
			player.abilitySelected = math.max(1, (player.abilitySelected + 1) % (#player.abilityInv + 1))
		end

		if key == settings.pickUp then -- Pick up an item
			local ctx, cty = math.floor(player.x / tileSize), math.floor(player.y / tileSize)
			if CURRENT_MAP == "map/Warehouse/Tutorial" and ((ctx == 24 and cty == 0) or (ctx == 25 and cty == 0)) then
				addProp({
					img = love.graphics.newImage("img/circle.png"),
					x = player.x + math.cos(math.pi*0.5) * tileSize,
					y = player.y + math.sin(math.pi*0.5) * tileSize,
					ttype = "proff",
					vel = 0,
					})

				player.notMoveTime = 3.5
				flux.to(player, 3.5, {notMoveTime = 0})

				RudeAwakeningString = ""
				local str = [[
Don't worry...
It will only sting a little...
]]
				flux.to(player, 0.1, {overlayOp = 255}):delay(0.5):ease("linear"):oncomplete(function() GLOBAL_STATE = "Rude Awakening"; stopBackgroundMusic() end):after(1, {overlayOp = 0}):ease("linear"):oncomplete(function() RudeAwakeningString = str; triggers.typeOutText = true end)
				--:after(1, {overlayOp = 255}):delay(4):ease("linear"):oncomplete(function() RudeAwakeningString = ""; flux.to(player, 0.1, {overlayOp = 0}) end):after(triggers, 4, {eyelidHeight = 128}):after(0.5, {eyelidHeight = 0}):oncomplete(function() RudeAwakeningString = "Rude Awakening" end):after(3, {eyelidHeight = h}):delay(0.5):ease("cubicout"):after(player, 1, {overlayOp = 255}):ease("linear"):oncomplete(function() GLOBAL_STATE = "game"; CURRENT_MAP = "map/Warehouse/1"; restart() end):after(0.5, {overlayOp = 0}):ease("linear")
			elseif CURRENT_MAP == "map/Warehouse/2" and ((ctx == 0 and cty == 0) or (ctx == 0 and cty == 1)) then
				player.notMoveTime = 1
				flux.to(player, 1, {notMoveTime = 0})
				triggers.eyelidHeight = 0

				RudeAwakeningString = ""
				local str = [[Now then. It is time for you to deliver on your side of the deal...
I need you to retrieve some documents from this address...]]

				flux.to(player, 1, {overlayOp = 255}):ease("linear"):oncomplete(function() GLOBAL_STATE = "Rude Awakening"; stopBackgroundMusic() end):after(0.1, {overlayOp = 0}):ease("linear"):oncomplete(function() RudeAwakeningString = str; triggers.typeOutText = true end)
			elseif math.dist(player.x, player.y, 330, 315) < player.w * 2 and not triggers.hasKey and not triggers.gotDocuments then
				pause = "cutscene"

				displayText(nil, {
					"The desk drawer is locked. A piece of paper lies on the desk. Something is scribbled on it.",
					"\"Note to self: I really must stop forgetting the drawer key on the night stand.\""
					})

				for i,v in ipairs(props) do
					if v.isObjective then
						table.remove(props, i)
					end
				end

				triggers.keyExists = true
				addProp({img = objectiveMarker, x = 1285, y = 278, vel = 0, w = objectiveMarker:getWidth(), h = objectiveMarker:getHeight(), displayR = love.math.random(0, math.pi), isObjective = true})
			elseif math.dist(player.x, player.y, 1285, 278) < player.w * 2 and triggers.keyExists then
				pause = "cutscene"

				displayText(nil, {"You pick up the key to the drawer.", "Go and collect the documents and get out."})

				for i,v in ipairs(props) do
					if v.isObjective then
						table.remove(props, i)
					end
				end

				triggers.keyExists = false
				triggers.hasKey = true
				addProp({img = objectiveMarker, x = 330, y = 315, vel = 0, w = objectiveMarker:getWidth(), h = objectiveMarker:getHeight(), displayR = love.math.random(0, math.pi), isObjective = true})
			elseif math.dist(player.x, player.y, 330, 315) < player.w * 2 and triggers.hasKey then
				pause = "cutscene"

				displayText(nil, {"You unlock the drawer. Inside are some important-looking papers.", "You catch a glimpse of the heading - \"CONTRACT\".", "You pocket the documents."})

				for i,v in ipairs(props) do
					if v.isObjective then
						table.remove(props, i)
					end
				end

				triggers.hasKey = false
				triggers.gotDocuments = true

				addProp({img = exitMarker, x = 2.5 * tileSize, y = 31.5 * tileSize, vel = 0, w = exitMarker:getWidth(), h = exitMarker:getHeight()})
			elseif math.dist(player.x, player.y, 80, 1008) < player.w * 3 and triggers.gotDocuments then
				player.notMoveTime = 1
				flux.to(player, 1, {notMoveTime = 0})
				triggers.eyelidHeight = 0

				local str = "Your success rang loud in my ears, loud like the hushed speculation of tommorow's day."
				if #enemies > (2 * totalEnemies / 3) then
					str = "Your success rang deep in my ears, though in no one else's. At least not today."
				end

				RudeAwakeningString = "Williamsburg, Brooklyn"
				flux.to(player, 1, {overlayOp = 255}):ease("linear"):oncomplete(function() GLOBAL_STATE = "Rude Awakening"; stopBackgroundMusic() end):after(2, {overlayOp = 0}):ease("linear"):delay(3):after(1, {overlayOp = 255}):delay(3):ease("linear"):oncomplete(function() RudeAwakeningString = "735 Driggs Avenue\n3rd Floor"; playSound("openLock") end):after(5.7, {overlayOp = 0}):ease("linear"):after(1.5, {overlayOp = 255}):delay(3):onstart(function() playSound("doorClose") end):ease("cubicout"):oncomplete(function() 
					flux.to(player, 0.1, {overlayOp = 0})

					RudeAwakeningString = ""
					pause = "cutscene"
					displayText(professor, {
						"My apologies for such an unwarranted intrusion. It seems I forgot to knock first.",
						str.." But I care not of the consequences.",
						"You have fulfilled your side of the deal, as have I. Our mutual obligations have come to an end.",
						"Perhaps, we will meet one another again. Perhaps... We shall meet someone different.",
						"I wonder...",
						})
				end)

			else
				local threw = false
				if player.inv[1].weapon and displayedProfessor then
					addProp({x = player.x, y = player.y, r = player.r, vel = 500}, {player.inv[1].weapon, player.inv[1].ammo})
					player.inv[1].weapon = nil
					player.inv[1].ammo = 0
					player.inv[1].btm = 0
					threw = true
				end

				local items

				if threw then
					items = world:queryRect(player:xx() - player.w * 0.5, player:yy() - player.h * 0.5, player.w * 2, player.h * 2, function(item) return item.isGun and (item ~= props[#props]) end)
				else
					items = world:queryRect(player:xx() - player.w * 0.5, player:yy() - player.h * 0.5, player.w * 2, player.h * 2, function(item) return item.isGun end)
				end

				if items[1] then
					--[[player.inv[1].weapon = items[1].isGun
					player.inv[1].ammo = items[1].ammo
					player.inv[1].btm = items[1].btm
					player.inv[1].bt = player.btm]]

					equipWeapon(items[1].isGun, 1, items[1].ammo)
					player.selected = 1

					world:remove(items[1])
					removeElement(props, items[1])

					if CURRENT_MAP == "map/Warehouse/Tutorial" and not displayedProfessor then
						addProp({
							img = love.graphics.newImage("img/circle.png"),
							x = player.x - math.cos(player.r) * tileSize,
							y = player.y - math.sin(player.r) * tileSize,
							ttype = "proff",
							vel = 0,
							})

						player.notMoveTime = 1
						flux.to(player, 1, {notMoveTime = 0})

						flux.to(props[#props], 1, {vel = 0}):oncomplete(function()
							pause = "cutscene"
							displayText(professor, {
								"You have accepted the conditions in the letter, then?", "...",
								"There is no need for haste. First, I need to verify your abilities.", "...",
								"Once you are done, exit at the back.",
								"(Use your left mouse button to shoot. Be careful, it's loud. Use middle-click or the space bar to melee - it doesn't make any noise.)",
								"(Use E to pick up weapons; and once you're done - to open the exit door)"}) end)
					end
				end
			end
		end
	end
elseif GLOBAL_STATE == "menu" then
	if key == "w" or key == "up" then
		menu.cur = menu.cur - 1; if menu.cur < 0 then menu.cur = menu.max - 1 end
	end
	if key == "s" or key == "down" then
		menu.cur = (menu.cur + 1) % menu.max
	end

	if key == "w" or key == "s" or key == "up" or key == "down" then
		if menu.state == "level" or menu.state == "main" then
			playSound("select")
		end
	end

	if key == "space" or key == "return" then
		activateMenu()
	end

	if key == "escape" then
		if menu.state == "level" or menu.state == "options" or menu.state == "credits" then
			menu.state = "main"
		elseif menu.state == "main" then
			love.event.quit()
		end
	end
elseif GLOBAL_STATE == "Rude Awakening" then
	if pause == "cutscene" then
		keyText(button)
	end
end
end

function love.wheelmoved(_, dy)
if GLOBAL_STATE == "game" then
	if not pause then
		if love.keyboard.isDown(settings.abilityInv) and settings.changeSlot == "mouseWheel" then
			
		elseif settings.changeSlot == "mouseWheel" then
			player.selected = player.selected + math.sign(dy)
			if player.selected > 4 then player.selected = 1 end
			if player.selected < 1 then player.selected = 4 end
		end
	end
elseif GLOBAL_STATE == "menu" then
end
end

function love.resize(w, h)
	push:resize(w, h)
end
