local binser = require "lib/binser"

mapBase 	= {} -- 2D
mapAI 		= {} -- 2D 
wallTable 	= {}
local propTable 	= {}
local enemyTable 	= {}

local text = ""
cutsceneText = {}
professor = love.graphics.newImage("img/professor.png")
gramophone = love.graphics.newImage("img/gramophone.png")

propList = {}

pathList = {}

IS_TUTORIAL = false
displayedProfessor = false

triggers = {
	r1 = false,
	r2 = false,
	r3 = false,

	r4 = false,
	r5 = false,
	r6 = false,

	eyelidHeight = 0,

	dtMult = 1,

	typeOutText = false,
	typeOutTimer = 1,
	typeOutString = 0,

	hasKey = false,
	keyExists = false,
	gotDocuments = false,
}

function getImgs(s, t)
	local lfs = love.filesystem
	local filesTable = lfs.getDirectoryItems(s)
	for i,v in ipairs(filesTable) do
		if lfs.isFile(s.."/"..v) then
			t[#t + 1] = love.graphics.newImage(s.."/"..v)
		end
	end
end

local function lines(str)
  local t = {}
  local function helper(line) table.insert(t, line) return "" end
  helper((str:gsub("(.-)\r?\n", helper)))
  return t
end

function generateMap(url, url2)
	pathList = {}
	propList = {}
	if url == "map/Warehouse/Tutorial" then
		IS_TUTORIAL = true
		displayedProfessor = displayedProfessor or false
	else
		IS_TUTORIAL = false
		displayedProfessor = true
	end

	local str = love.filesystem.read(url.."/".."data")
	local ps
	mapBase, mapAI, wallTable, propTable, enemyTable, mapW, mapH, ps, text = binser.deserializeN(str, 9)
	--print(text)
	cutsceneText = lines(text)
	cutsceneText.curVal = 1

	camera:lookAt((ps.x + mapW * tileSize * 0.5) * 0.5, (ps.y + mapH * tileSize * 0.5) * 0.5)

	if cutsceneText[1] ~= "" and (not displayedProfessor or not IS_TUTORIAL) then
		pause = "cutscene"

		if url == "map/Warehouse/2" and triggers.r4 then
			displayText(nil, "Try again - hold right-click to use your selected ability.")
		elseif url == "map/Warehouse/3" and triggers.r6 then
			displayText(nil, "Just get the documents and get out - simple.")
		else
			displayText(nil, cutsceneText)

			if url == "map/Warehouse/2" then triggers.r4 = true end
			if url == "map/Warehouse/3" then triggers.r6 = true end
		end
	end

	getImgs(url.."/".."props", propList)
	generatePathfindingMap()

	for i,v in ipairs(propTable) do
		v.vel = v.vel or 0
		addProp(v)
	end

	if url == "map/Warehouse/Tutorial" then
		addProp({x = 125, y = 535, r = math.pi/5, vel = 0}, {"pistol", 7})
		addProp({x = 430, y = 610, r = math.pi/3.5, vel = 0}, {"auto", 20})
		addProp({x = 725, y = 215, r = math.deg(12), vel = 0}, {"shotgun", 10})
	end

	for i,v in ipairs(enemyTable) do
		v.ammo = (v.weapon == "knife") and 0 or guns[v.weapon][3]
		v.btm = (v.weapon == "auto") and 0.1 or 0.5
		v.run = (v.weapon == "knife" and 200 or 100)

		v.img = nil

		v.tutorial = IS_TUTORIAL

		table.insert(enemies, en.new(v))

		if IS_TUTORIAL then
			table.insert(pathList, v.path)
		end

		if v.mode ~= "stand" then
			en.embark(enemies[#enemies], v.path, nil, v.loop)
		end
	end

	for i,v in ipairs(wallTable) do
		v.type = "wall"
		world:add(v, v.x, v.y, v.w, v.h)
	end

	--[[local mData = love.image.newImageData(url)
	mapW, mapH = mData:getDimensions()
	for y=0, mData:getHeight() - 1 do
		map[y] = {}
		for x=0, mData:getWidth() - 1 do
			local _, _, _, a= mData:getPixel(x, y)
			if a ~= 0 then
				map[y][x] = 0
			else
				map[y][x] = 1
			end
		end
	end]]

	

	--[[ Enemy spawn
	local nData = love.image.newImageData(url2)
	local colourArray = {}
	for y=0, nData:getHeight() - 1 do
		for x=0, nData:getWidth() - 1 do
			local r, g, b, a = nData:getPixel(x, y)

			if a ~= 0 then
				if colourArray[r..g..b] then
					local v = colourArray[r..g..b]

					local weapon = love.math.random(1,4)
					local wepStr = weapon == 1 and "auto" or weapon == 2 and "shotgun" or weapon == 3 and "pistol" or "knife"

					table.insert(enemies, en.new({
						x = (v.x + 0.5) * tileSize, y = (v.y + 0.5) * tileSize,
						weapon = wepStr, ammo = (wepStr == "knife") and 0 or guns[wepStr][3],
						btm = (wepStr == "auto") and 0.1 or 0.5,
						run = (wepStr == "knife" and 200 or 100)
					}))
					en.embark(enemies[#enemies], generatePath(v.x, v.y, x, y), nil, true)
				else
					colourArray[r..g..b] = {x = x, y = y}
				end
			end
		end
	end]]

	for y = 0, mapH - 1 do
		for x = 0, mapW - 1 do
			if checkMap(x, y) == 0 then
				if checkMap(x - 1, y) ~= 0 then
					world:add({x = x - 1, y = y, type = "wall"}, (x - 1) * tileSize, y * tileSize, tileSize, tileSize)
				end

				if checkMap(x + 1, y) ~= 0 then
					world:add({x = x + 1, y = y, type = "wall"}, (x + 1) * tileSize, y * tileSize, tileSize, tileSize)
				end

				if checkMap(x, y - 1) ~= 0 then
					world:add({x = x, y = y - 1, type = "wall"}, x * tileSize, (y - 1) * tileSize, tileSize, tileSize)
				end

				if checkMap(x, y + 1) ~= 0 then
					world:add({x = x, y = y + 1, type = "wall"}, x * tileSize, (y + 1) * tileSize, tileSize, tileSize)
				end
			end
		end
	end

	totalEnemies = #enemies

	return ps.x, ps.y
end

local myFinder
function generatePathfindingMap()
	local walkable = 0

	-- Library setup
	local Grid = require ("lib.jumper.grid") -- The grid class
	local Pathfinder = require ("lib.jumper.pathfinder") -- The pathfinder lass

	-- Creates a grid object
	local grid = Grid(mapAI) 
	-- Creates a pathfinder object using Jump Point Search
	myFinder = Pathfinder(grid, 'ASTAR', walkable)
	myFinder:setMode('ORTHOGONAL')
end

function generatePath(x, y, dx, dy, count)
	count = count or 0

	if limitMap(x, y) then
		if mapAI[y][x] ~= 0 then
			x = x + love.math.random(-2, 2)
			y = y + love.math.random(-2, 2)
		end
	end

	if limitMap(x, y) and limitMap(dx, dy) then
		local path = myFinder:getPath(x, y, dx, dy)

		local ep = {}

		if path then
			for node, count in path:nodes() do
				ep[#ep + 1] = {x = (node:getX() + 0.5) * tileSize, y = (node:getY() + 0.5) * tileSize}
			end
		else
			return nil
		end

		return ep
	else
		if count > 40 then
			return {{x = x, y = y}}
		else
			return generatePath(x, y, math.max(0, math.min(mapW - 1, dx + love.math.random(-4, 4))), math.max(0, math.min(mapH - 1, dy + love.math.random(-4, 4))), count + 1)
		end
	end
end

function checkMap(x, y)
	if mapBase[y] then
		if mapBase[y][x] then
			return mapBase[y][x]
		else
			return -1
		end
	end

	return -2
end

function limitMap(x, y)
	return not (x < 0 or x > mapW - 1 or y < 0 or y > mapH - 1)
end

function reversePath(path)
	local p1 = {}
	for i = #path, 1, -1 do
		p1[#p1 + 1] = path[i]
	end

	return p1
end