function removeElement(main, j)
	table.remove(main, lookUpID(main, j))
end

function lookUpID(main, j)
	for i,v in ipairs(main) do
		if v == j then return i end
	end

	print("-1")
	return -1
end

function stripes()
	for x = 0, w * 2, 20 do
		for y = 0, h * 1.5 do
			love.graphics.setColor(255, 255, 255)
			love.graphics.line(x - y / 2, y, x + 3 - y / 2, y)

			love.graphics.setColor(0, 0, 0)
			love.graphics.points(x - y / 2 + love.math.random(0, 3), y)
		end
	end
end

function printText(str, x, y)
	love.graphics.setColor(0, 0, 0)
	--[[love.graphics.print(str, x-1, y)
	love.graphics.print(str, x+1, y)
	love.graphics.print(str, x, y-1)
	love.graphics.print(str, x, y+1)]]
	love.graphics.rectangle("fill", x, y, font:getWidth(str), font:getHeight())
	love.graphics.setColor(255, 255, 255)
	love.graphics.print(str, x, y)
end

--[[local function checkPos(x, y)
	return map[y] and map[y][x] == 0 or false
end

local function checkMove(x, y, dx, dy, count, arr)
	if checkPos(x + dx, y + dy) and count >= 0 then
		x, y = x + dx, y + dy
		for i, v in ipairs(arr) do
			if v.x == x and v.y == y then break end
		end
		table.insert(arr, {x = x, y = y})

		checkMove(x, y, 	1, 	0, count - 1, arr)
		checkMove(x, y,    -1, 	0, count - 1, arr)
		checkMove(x, y, 	0, 	1, count - 1, arr)
		checkMove(x, y, 	0, -1, count - 1, arr)
	end
end

function generateVisibleTiles(x, y, r) -- Return a table of tiles where the explosion should be visible
	local arr = {}

	print(math.ceil(r / tileSize))
	checkMove(x, y, 0, 0, math.ceil(r / tileSize), arr)

	return arr
end

local stencilArray = {}

function setStencilForGrenade(arr)
	stencilVisible = arr
end]]--

function stencil()
	love.graphics.setColor(255, 255, 255)
	for y = 0, mapH - 1 do
		for x = 0, mapW - 1 do
			if checkMap(x, y) == 0 then
				love.graphics.rectangle("fill", x * tileSize, y * tileSize, tileSize, tileSize)
			end
		end
	end
end

explosions = {}

function addExplosion(x, y, r, t)
	table.insert(explosions, {x = x, y = y, r = 0, t = t})
	flux.to(explosions[#explosions], 0.1, {r = r}):ease("quadout"):after(0.05, {r = 0}):ease("quadin"):delay(0.1)
end

function updateExplosion(dt)
	for i,v in ipairs(explosions) do
		if v.r == 0 then
			table.remove(explosions, i)
		end
	end
end

function drawExplosion()
	love.graphics.setColor(255, 255, 255)

	for i,v in ipairs(explosions) do	
		--setStencilForGrenade(v.t)
		if not v.t then
			love.graphics.stencil(stencil, "replace", 1)
			love.graphics.setStencilTest("greater", 0)

			love.graphics.circle("fill", v.x, v.y, v.r)

			love.graphics.setStencilTest()
		else
			love.graphics.setLineWidth(love.math.random(0, 4))
			love.graphics.circle("line", v.x, v.y, v.r)
			love.graphics.setLineWidth(2)
		end
	end
end

local sfx = {
	rot = math.pi / 100
}

-- For rotating background and level
function tweenRotA(t)
	local t = t or 4
	flux.to(camera, t, {rotation = sfx.rot}):oncomplete(tweenRotB):ease("linear")
end
function tweenRotB()
	flux.to(camera, 4, {rotation = -sfx.rot}):oncomplete(tweenRotA):ease("linear")
end

function tweenRotAA(t)
	local t = t or 4
	flux.to(back, t, {r = sfx.rot}):oncomplete(tweenRotBB):ease("linear")
end
function tweenRotBB()
	flux.to(back, 4, {r = -sfx.rot}):oncomplete(tweenRotAA):ease("linear")
end

local tw, ms, mr, pc = nil
function lightbulbFlicker()
	if GLOBAL_STATE == "menu" then
		if not ms then ms = playSound("lightbulb") end
		if not mr then mr = playSound("rain") end

		menu.c = color[cRef[love.math.random(1, 7)]]
		tw = flux.to(menu, 2, {a = 255}):ease("bouncein"):after(2, {a = 0}):ease("bounceinout"):delay(love.math.random(2,4)):after(love.math.random(2,5), {a=0}):oncomplete(lightbulbFlicker)
	end
end

function panCamera()
	if GLOBAL_STATE == "menu" then
		pc = flux.to(menu, 5, {panCam = 8}):ease("quadinout"):after(10, {panCam = -8}):ease("quadinout"):oncomplete(panCamera)
	end
end

function stopCameraPan()
	pc:stop()
end

function lightbulbStop()
	tw:stop()
	stopSound("lightbulb", ms)
	stopSound("rain", mr)
	ms, mr = nil, nil
end

function lightbulbVol(vol)
	setVolume("lightbulb", ms, vol)
end

function newRainMesh()
	local o = love.math.random(0,1)
	local vertices = {
		{ 0, 0, o, o, 255, 255, 255, 255 },
		{ menu.rain:getWidth(), 0, 1 + o, o, 255, 255, 255, 255 },
		{ menu.rain:getWidth(), menu.rain:getHeight(), 1 + o, 1 + o, 255, 255, 255, 255 },
		{ 0, menu.rain:getHeight(), o, 1 + o, 255, 255, 255, 255 },
	}

	local mesh = love.graphics.newMesh(vertices, 'fan')
	mesh:setTexture(menu.rain)

	return mesh
end

currentlyPlaying = {
	name = "",
	id = nil,
}

function playBackgroundMusic(sound)
	stopBackgroundMusic()
	currentlyPlaying.name = sound
	currentlyPlaying.id = playSound(sound)
end

function stopBackgroundMusic()
	if currentlyPlaying.id then
		stopSound(currentlyPlaying.name, currentlyPlaying.id)
	end
end

function goBackToMenu()
	GLOBAL_STATE = "menu"

	displayedProfessor = false

	triggers = {
		r1 = false,
		r2 = false,
		r3 = false,

		r4 = false,
		r5 = false,

		eyelidHeight = 0,

		dtMult = 1,

		typeOutText = false,
		typeOutTimer = 1,
		typeOutString = 0,
	}

	playBackgroundMusic("mainTheme")
	lightbulbFlicker()
	panCamera()
end

function activateMenu()
	playSound("talk")

	if menu.state == "main" then
		if menu.cur == 0 then
			menu.state = "level"
		elseif menu.cur == 1 then
			love.system.openURL("file://"..love.filesystem.getWorkingDirectory( ).."/editor/LevelEditor_x86.exe")
		elseif menu.cur == 2 then
			menu.state = "options"
		elseif menu.cur == 3 then
			menu.state = "credits"
		elseif menu.cur == menu.max - 1 then
			love.event.quit()
		end
	elseif menu.state == "level" then
		if menu.cur == 0 then
			GLOBAL_STATE = "Rude Awakening"
			player.overlayOp = 255
			RudeAwakeningString = [[Brooklyn, 1930
The Docks]]
			flux.to(player, 3, {overlayOp = 0}):after(2, {overlayOp = 255}):delay(4):ease("linear"):oncomplete(function() GLOBAL_STATE = "game"; CURRENT_MAP = "map/Warehouse/Tutorial"; restart(); playBackgroundMusic("mainTheme") end):after(1, {overlayOp = 0}):ease("linear")
		elseif menu.cur == 1 then
			GLOBAL_STATE = "Rude Awakening"
			player.overlayOp = 0
			RudeAwakeningString = ""
			flux.to(triggers, 4, {eyelidHeight = 128}):delay(1):after(0.5, {eyelidHeight = 0}):oncomplete(function() RudeAwakeningString = "Rude Awakening" end):after(3, {eyelidHeight = h}):delay(0.5):ease("cubicout"):after(player, 1, {overlayOp = 255}):ease("linear"):oncomplete(function() GLOBAL_STATE = "game"; CURRENT_MAP = "map/Warehouse/2"; restart(); playBackgroundMusic("fightTheme") end):after(0.5, {overlayOp = 0}):ease("linear"):oncomplete(function() triggers.eyelidHeight = 0 end)
		elseif menu.cur == 2 then
			GLOBAL_STATE = "Rude Awakening"
			player.overlayOp = 255
			RudeAwakeningString = "Upstate New York, 1930"

			flux.to(player, 3, {overlayOp = 0}):ease("linear"):after(1, {overlayOp = 255}):delay(3):ease("linear"):oncomplete(function() RudeAwakeningString = "The Agreement" end):after(2, {overlayOp = 0}):ease("linear"):delay(2):after(1, {overlayOp = 255}):delay(3):oncomplete(function() GLOBAL_STATE = "game"; CURRENT_MAP = "map/Warehouse/3"; restart(); playBackgroundMusic("fightTheme") end):after(1, {overlayOp = 0}):ease("linear")
		elseif menu.cur == menu.max - 1 then
			menu.state = "main"
		end

		if menu.cur >= 0 and menu.cur <= 2 then
			lightbulbStop()
			stopCameraPan()
			stopBackgroundMusic()

			player.notMoveTime = 1
			flux.to(player, 1, {notMoveTime = 0})

			love.mouse.setGrabbed(true)
		end

	elseif menu.state == "options" or menu.state == "credits" then
		menu.state = "main"
	end

	if menu.state == "main" then
		menu.max = 5
	else
		menu.max = 4
	end
end