objectiveMarker = love.graphics.newImage("img/misc/objective.png")
exitMarker = love.graphics.newImage("img/misc/exit.png")

function killPlayer(m, a, vel)
	vel = vel or 500
	addProp({x = m.x, y = m.y, r = a, vel = vel})
	camera:split(8, 0.05)

	if player.inv[1].weapon then
		addProp({x = player.x, y = player.y, r = a,  displayR = a - math.pi * 0.5, vel = vel * 1.5}, {player.inv[1].weapon, player.inv[1].weapon.ammo})
	end
	addProp({x = player.x, y = player.y, r = a, displayR = 0, vel = vel * 0.75, img = love.graphics.newImage("img/misc/heart.png"), c = HSL(player.heart.c, 255, 128)})
	playSound("perished")

	-- RESTART THE WORLD
	pause = true
	pauseText = "DEAD"
	deadTimer = 0.5

	stopAbilitySound()
	
	world:remove(player)
end

function restart()
	world = bump.newWorld(32)
	enemies = {}
	props = {}
	enBullets = {}

	wipeParticles()
	stopAbilitySound()
	--map = {}
	--generateMap("map/map5.png", "map/npc5.png")
	player.x, player.y = generateMap(CURRENT_MAP)

	player.bullets = {}
	player.abilityDrain = 0

	shaders.glow.radius = player.abilityDrain * 4
	shaders.grain.opacity = player.abilityDrain

	triggers.hasKey = false
	triggers.keyExists = false

	if displayedProfessor and CURRENT_MAP == "map/Warehouse/Tutorial" then
		world:remove(wallTable[#wallTable - 1]); table.remove(wallTable, #wallTable - 1)
	end

	if triggers.r3 and CURRENT_MAP == "map/Warehouse/Tutorial" then
		world:remove(wallTable[#wallTable]); table.remove(wallTable, #wallTable)
	end

	if IS_TUTORIAL or CURRENT_MAP == "map/Warehouse/2" then
		player.inv = {
				[1] = {
					weapon = nil,
				}, -- corresponds to weapon, usually
				[2] = {
					grenade = "explode",
					ammo = 0,
				},
				[3] = {
					grenade = "noise",
					ammo = 0,
				},
				[4] = {
					grenade = "stun",
					ammo = 0,
				},
			}
		player.inv[1] = { weapon = nil }
		player.abilityInv = {}
		equipWeapon("explode", 2, 2)

		if CURRENT_MAP == "map/Warehouse/2" then
			player.inv = {
				[1] = {
					weapon = nil,
				}, -- corresponds to weapon, usually
				[2] = {
					grenade = "explode",
					ammo = 0,
				},
				[3] = {
					grenade = "noise",
					ammo = 0,
				},
				[4] = {
					grenade = "stun",
					ammo = 0,
				},
			}
			player.inv[1] = { weapon = nil }
			player.abilityInv = {
				{"spatial dilation", nil},
				{"psyche wall", nil},
			}
		end

		if triggers.r5 then
			en.die(enemies[#enemies], player.r, true, true)
		end
	else
		player.abilityInv = {
			{"spatial dilation", nil},
			{"psyche wall", nil},
		}

		equipWeapon("pistol", 1)
		equipWeapon("explode", 2)
		equipWeapon("noise", 3, 2)
		equipWeapon("stun", 4, 5)
	end

	if CURRENT_MAP == "map/Warehouse/3" then
		addProp({img = objectiveMarker, x = 330, y = 315, vel = 0, w = objectiveMarker:getWidth(), h = objectiveMarker:getHeight(), displayR = love.math.random(0, math.pi), isObjective = true})
	end

	player.selected = 1
	player.abilitySelected = 1
	world:add(player, player:xx(), player:yy(), player.w, player.h)
	
	player.selected = 1
end

function shoot()
	local x, y = camera:mousePosition()
	local ang = math.angle(player.x, player.y, x, y)

	local s = player.inv[player.selected]

	if s.weapon and s.ammo > 0 then
		camera:shake(4, 0.05)

		if s.weapon == "pistol" then
			playSound("shoot")
		elseif s.weapon == "auto" then
			playSound("machinegun")
		elseif s.weapon == "shotgun" then
			playSound("shotgun")
		end

		if s.weapon == "shotgun" then
			for i = 0, 4 do
				table.insert(player.bullets, {x = player.x, y = player.y, a = ang + math.rad(love.math.random(-15, 15)), v = 3000})
			end
		else
			table.insert(player.bullets, {x = player.x, y = player.y, a = ang + (player.weapon == "auto" and math.rad(love.math.random(-4, 4)) or 0), v = 3000})
		end

		en.canHearPlayer(enemies, guns[s.weapon][4])

		s.ammo = s.ammo - 1
	elseif s.grenade and s.ammo > 0 then -- Add different grenades
		local isMoving = love.keyboard.isDown("w", "s", "a", "d")

		table.insert(player.bullets, {
			x = player.x, y = player.y,
			blowRad = grenades[s.grenade][2],
			vx = math.cos(ang), vy = math.sin(ang),
			v = 500 + (isMoving and player.speed or 0), type = "prop",
			grenadeType = s.grenade
		})
		world:add(player.bullets[#player.bullets], player.x, player.y, 1, 1)
		flux.to(player.bullets[#player.bullets], 2, {v = 0}):ease("quadout")

		s.ammo = s.ammo - 1
	end
end

function slash()
	playSound("slash")
	player.slash = flux.to(player, 0.1, {slashr = -math.rad(150) * 0.5}):ease("quadout"):oncomplete(function() player.slash = nil; player.slashr = math.rad(150) * 0.5 end)

	for i,v in ipairs(enemies) do
		if math.dist(v.x, v.y, player.x, player.y) <= player.h * 1.5 + v.h * 0.5 then
			if math.abs(angleDifference(player.r, math.angle(player.x, player.y, v.x, v.y))) < math.rad(150) * 0.5 then
				en.die(v, player.r, true, true)
				playSound("perished")
			end
		end
	end
end