local text = {}

local function doubleText(t, x, y, limit) -- Set this outer text to white and the inner to black
	love.graphics.printf(t, 	x, 		y - 2, 	limit)
	love.graphics.printf(t, 	x, 		y + 2,  limit)
	love.graphics.printf(t, 	x - 2, 	y, 		limit)
	love.graphics.printf(t, 	x + 2, 	y, 		limit)
end

local function finishTalking()
	local willContinue = false
	if text.index then
		willContinue = text.index < #text.t
	end

	text.doneTalking = true
	text.time = 0

	if not willContinue then
		flux.to(text, 0.5, {time = 1})
	end
	
	text.continue = willContinue
end

function displayText(icon, t, speed) -- t is script table, it has text and choices if there are any
	speed = speed or 20

	text = {
		img = icon,
		s = "",
		t = t,
		v = speed,
		time = 0,
		active = true,

		doneTalking = false,
	}
	
	if type(text.t) == "table" then
		text.index = 1 
	end
end

function updateText(dt)
	if not text.doneTalking and text.v then
		local fullText = (text.index) and text.t[text.index] or text.t

		text.time = text.time + text.v * dt
		if text.time >= 1 then
			text.time = 0

			if text.img then
				playSound("talkDeep", true)
			else
				playSound("talk", true)
			end
			text.s = string.sub(fullText, 1, string.len(text.s) + 1)
		end

		if string.len(text.s) == string.len(fullText) then
			finishTalking()
		end
	end
end

local width = 1280
local height = 160

function drawText()
		--love.graphics.setFont(font32)
		love.graphics.setColor(0, 0, 0)
		love.graphics.rectangle("fill", (w - width) * 0.5 - 1, h - height - 1, width, height)
		love.graphics.rectangle("fill", 0, h - height, w, height)
		love.graphics.setColor(255, 255, 255)
		--love.graphics.rectangle("line", (w - width) * 0.5 - 1, h - height - 1, width, height)

		--love.graphics.setScissor((w - width) * 0.5, h - height, height - 2, height - 2)
		--love.graphics.draw(text.img, (w - width) * 0.5, h - height, 0, (height - 2) / text.img:getHeight(), (height - 2) / text.img:getHeight())
		if text.img then
			love.graphics.draw(text.img, 0, h - text.img:getHeight())
		end
		--love.graphics.setScissor()
		
		love.graphics.setColor(255, 255, 255)
		local www = text.img and text.img:getWidth() + height * 0.25 or height
		love.graphics.printf(text.s, www, h - height + 5, w - www)

		--love.graphics.setFont(font16)

	love.graphics.setColor(255, 255, 255)
end

function keyText(k)
	local fullText = (text.index) and text.t[text.index] or text.t

	if k and string.len(text.s) ~= string.len(fullText) then
		text.s = fullText
		finishTalking()
	elseif text.doneTalking and text.continue then
		text.index = text.index + 1

		text.s = ""

		text.doneTalking = false
		text.time = 0
	elseif text.doneTalking and not text.continue then
		pause = false
		local notMove = true

		if not displayedProfessor and props[#props].ttype == "proff" and CURRENT_MAP == "map/Warehouse/Tutorial" then
			local p = generatePath(math.floor(props[#props].x/tileSize), math.floor(props[#props].y/tileSize), 5, 13)
			flux.to(props[#props], 0.5, {x = p[1].x, y = p[1].y}):ease("linear")
			for i=2,(#p-1) do
				flux.to(props[#props], 0.5, {x = p[i].x, y = p[i].y}):ease("linear"):delay((i-1)*0.5)
			end
			flux.to(props[#props], 0.5, {x = p[#p].x, y = p[#p].y}):ease("linear"):delay((#p-1)*0.5):after(0.3, {r = 0}):oncomplete(function() world:remove(wallTable[#wallTable - 1]); table.remove(wallTable, #wallTable - 1) end):after(0.5, {x = 5.5 * tileSize, y = 14.5 * tileSize}):ease("linear")

			displayedProfessor = true
		elseif CURRENT_MAP == "map/Warehouse/2" and (text.s == "(Last message)" or text.s == "Try again - hold right-click to use your selected ability.") then
			notMove = false

			triggers.dtMult = triggers.r5 and 1 or 1/10
			flux.to(triggers, 5, {eyelidHeight = h}):oncomplete(function() triggers.dtMult = 1 end)
		elseif text.s == "I wonder..." then
			pause = "cutscene"
			flux.to(player, 4, {overlayOp = 255}):ease("linear"):oncomplete(function() RudeAwakeningString = "Thank you for playing."; pause = false end):after(1, {overlayOp = 0}):delay(2):after(1, {overlayOp = 255}):delay(4):oncomplete(function() goBackToMenu() end):after(1, {overlayOp = 0}):ease("linear"):delay(3)
		end

		if notMove then
			player.notMoveTime = 0.3
			flux.to(player, 0.3, {notMoveTime = 0})
		end
	end
end